#include <PalmOS.h>
#include <Hs.h>
#include <PalmOnePhoto.h>
#include <HsPhonePower.h>
#include <HsPhoneNetwork.h>
#include <HsPhoneLibrary.h>
#include <HsPhoneAudioTypes.h>
#include <HsPhoneGSM.h>
#include <PmSysGadgetLib.h>
#include <HsKeyCodes.h>
#include <BtLib.h>
#include <BtPrefsPnlTypes.h>

#include "NeoLauncher.h"
#include "NeoLauncher_Rsc.h"
#include "Setup.h"
#include "NongLi.h"

static void drawPhoneUIBackground(SettingType *settingP, GlobeType *globeP);
static void drawPhoneUIStatusBar(SettingType *settingP, GlobeType *globeP);
static void drawPhoneUI(UInt16 code, SettingType *settingP, GlobeType *globeP);
static Boolean frmPhoneUICustomEventHandler(EventType *eventP, SettingType *settingP, GlobeType *globeP);

static Boolean isRunningOnTreo(SettingType *settingP);

static void launchProgram(ProgramListType *programInfo, UInt16 selectedProgram, SettingType *settingP, GlobeType *globeP);
static void paintCharWithOutline(Char chr, Coord x, Coord y, Boolean shouldDrawOutline, RGBColorType *outlineColorP, GlobeType *globeP);
static void paintCharsWithOutline(const Char *stringP, UInt16 stringLength, Coord x, Coord y, Boolean shouldDrawOutline, RGBColorType *outlineColorP, GlobeType *globeP);
static void winPaintTruncChars(Char *stringP, UInt16 stringLength, Coord x, Coord y, UInt16 maxWidth, Boolean shouldDrawOutline, RGBColorType *outlineColorP, GlobeType *globeP);
static void convertCharsToBitPackage(Char *srcStringP, UInt32 *destUInt32BitPackageP, UInt16 *UInt32BitPackageLengthP);
static Char *strPreventChr(Char *stringP, Char chr);
static void freeGlobeCategoryList(GlobeType *globeP);
static void freeGlobeProgramList(GlobeType *globeP);

static void loadTinyFont(GlobeType *globeP);
static void unloadTinyFont(GlobeType *globeP);

#if 0
#pragma mark -
#pragma mark GLOBE VARIBLES
#pragma mark -
#endif

#if 0
#pragma mark -
#pragma mark PROGRAM LIST FUNCTIONS
#pragma mark -
#endif

#ifdef DEBUG_LOG
static void printLogMessage(Char *messageP)
{
    Err                     error           = errNone;
    FileHand                file;

    file = FileOpen( 0, "NL_Log.csv", sysFileTFileStream, 0, fileModeAppend, &error );
    if ( error == errNone )
    {
        FileWrite( file, messageP, StrLen(messageP), 1, &error  );
        FileWrite( file, "\r\n", StrLen( "\r\n" ), 1, &error );

        FileClose( file );
    }
}
#endif /* DEBUG_LOG */

//ZLĻص
static void NeoZLReopen(ZLPluginLaunchCmdBlockType * blockP)
{
    ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;

    if (blockExP->pluginDataP != NULL)
    {
        ZLPluginData *dataP = (ZLPluginData *)blockExP->pluginDataP;

        dataP->ZLPluginFuncPtrs[ZLPluginFuncPtrClose]((ZLPluginLaunchCmdBlockType *)&(dataP->cmdBlockTypeEx));
        dataP->ZLPluginFuncPtrs[ZLPluginFuncPtrOpen]((ZLPluginLaunchCmdBlockType *)&(dataP->cmdBlockTypeEx));
    }

    return;
}

static void NeoZLBeginDraw(ZLPluginLaunchCmdBlockType * blockP, Boolean lockScreen)
{
    return;
}

static void NeoZLEndDraw(ZLPluginLaunchCmdBlockType * blockP)
{
    return;
}

static void NeoZLPrepareHandlingEvent(ZLPluginLaunchCmdBlockType * blockP)
{
    return;
}

static void NeoZLSetHint(ZLPluginLaunchCmdBlockType * blockP, Char * hints)
{
    return;
}

static void NeoZLShowHint(ZLPluginLaunchCmdBlockType * blockP, Coord x, Coord y)
{
    return;
}

static void NeoZLHideHint(ZLPluginLaunchCmdBlockType * blockP)
{
    return;
}

static UInt32 NeoZLPrefsReadValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, UInt32 defaultValue)
{
    ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;
    UInt32 value;
    UInt16 size = sizeof(UInt32);

    if (PrefGetAppPreferences(blockExP->creatorId, id, &value, &size, true) != noPreferenceFound)
        return value;

    return defaultValue;
}

static UInt32 NeoZLPrefsReadVariableValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, void * valueP, UInt16 * sizeP)
{
    ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;

    if (PrefGetAppPreferences(blockExP->creatorId, id, valueP, sizeP, true) != noPreferenceFound)
        return *sizeP;

    return noPreferenceFound;
}

static UInt16 NeoZLPrefsWriteValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, UInt32 value)
{
    ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;
    UInt16 size = sizeof(value);

    PrefSetAppPreferences(blockExP->creatorId, id, 1, &value, size, true);

    return size;
}

static UInt16 NeoZLPrefsWriteVariableValue(ZLPluginLaunchCmdBlockType * blockP, UInt16 id, void * valueP, UInt16 size)
{
    ZLPluginLaunchCmdBlockTypeEx *blockExP = (ZLPluginLaunchCmdBlockTypeEx *)blockP;

    PrefSetAppPreferences(blockExP->creatorId, id, 1, valueP, size, true);

    return size;
}

static void NeoZLRefreshBackground(ZLPluginLaunchCmdBlockType * blockP)
{
    return;
}

static void NeoZLDrawChars(ZLPluginLaunchCmdBlockType * blockP, Char * chars, Coord x, Coord y, Coord width)
{
    WinPaintChars(chars, StrLen(chars), x, y);
}

static void NeoZLDrawBitmap(ZLPluginLaunchCmdBlockType * blockP, BitmapType * pBitmap, Coord x, Coord y)
{
    WinPaintBitmap(pBitmap, x, y);
}

//Ƿд濨
static Boolean isCardInserted()
{
    UInt16          volRef          = 0;
    UInt32          volIterator     = vfsIteratorStart;

    while (volIterator != vfsIteratorStop)
    {
        VFSVolumeEnumerate(&volRef, &volIterator);
    }

    if (volRef > 0)
    {
        return true;
    }

    return false;
}

//ȫֱĳб
static void freeGlobeProgramList(GlobeType *globeP)
{
    if (globeP->programListP != NULL)
    {
        MemPtrFree(globeP->programListP);

        globeP->programListCount = 0;
        globeP->programListP = NULL;
    }
}

//ȫֱķб
static void freeGlobeCategoryList(GlobeType *globeP)
{
    if (globeP->categoryListPP)
    {
        UInt16      i;

        for (i = 0; i < globeP->categoryCount; i ++)
        {
            if (globeP->categoryListPP[i] != NULL)
                MemPtrFree(globeP->categoryListPP[i]);
        }
        if (globeP->categoryListPP != NULL)
            MemPtrFree(globeP->categoryListPP);
        globeP->categoryCount = 0;
    }

    if (globeP->categoryListMappingP)
    {
        MemPtrFree(globeP->categoryListMappingP);
    }

    if (globeP->hideCategoryP)
    {
        MemPtrFree(globeP->hideCategoryP);
    }
    globeP->hideCategoryCount = 0;

    globeP->categoryListPP = NULL;
    globeP->categoryListMappingP = NULL;
    globeP->hideCategoryP = NULL;
}

//ȡȫ
static void buildCategoryList(GlobeType *globeP)
{
    UInt16          i, recordNums;
    MemHandle       recordH;
    CategoryType    *recordP;
    DmOpenRef       dbRef       = DmOpenDatabaseByTypeCreator(CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadOnly);

    //ͷз
    freeGlobeCategoryList(globeP);

    if (dbRef != NULL)
    {
        //
        recordNums = DmNumRecordsInCategory(dbRef, dmAllCategories);
        i = recordNums + 2;
        globeP->categoryListPP = (Char **)MemPtrNew(i * 4);
        globeP->categoryListMappingP = (UInt8 *)MemPtrNew(i);
        MemSet(globeP->categoryListMappingP, i, 0xFF);
        globeP->hideCategoryP = (UInt8 *)MemPtrNew(i);
        MemSet(globeP->hideCategoryP, i, 0xFF);

        //ȫ
        globeP->categoryListPP[0] = (Char *)MemPtrNew(StrLen(STRING_ALL_CATEGORIER) + 1);
        StrCopy(globeP->categoryListPP[0], STRING_ALL_CATEGORIER);
        globeP->categoryListMappingP[0] = ALL_CATEGORIES;
        globeP->categoryCount = 1;
        globeP->hideCategoryCount = 0;

        for(i = 0; i < recordNums; i ++)
        {
            recordH = DmQueryRecord(dbRef, i);
            if (recordH == NULL)
                break;
            recordP = (CategoryType *)MemHandleLock(recordH);
            if (! recordP->isCategoryHide)
            {
                globeP->categoryListPP[globeP->categoryCount] = (Char *)MemPtrNew(StrLen(recordP->nameP) + 1);
                StrCopy(globeP->categoryListPP[globeP->categoryCount], recordP->nameP);
                globeP->categoryListMappingP[globeP->categoryCount] = recordP->categoryId;
                globeP->categoryCount ++;
            }
            else
            {
                globeP->hideCategoryP[globeP->hideCategoryCount] = recordP->categoryId;
                globeP->hideCategoryCount ++;
            }
            MemHandleUnlock(recordH);
        }

#if 0
        //濨
        if (isCardInserted())
        {
            globeP->categoryListPP[globeP->categoryCount] = (Char *)MemPtrNew(StrLen(STRING_CARD) + 1);
            StrCopy(globeP->categoryListPP[globeP->categoryCount], STRING_CARD);
            globeP->categoryListMappingP[globeP->categoryCount] = CARD_CATEGORY;
            globeP->categoryCount ++;
        }
#endif
        DmCloseDatabase(dbRef);
    }
}

//ȡķ
static void getProgramCategory(ProgramListType *dbInfoP)
{
    Char                *rscP;
    MemHandle           rscH;
    CategoryType        *recordP;
    MemHandle           recordH;
    UInt16              recordId;
    UInt16              i;
    UInt16              recordNums;
    UInt8               maxCategoryId;
    DmOpenRef           cDbRef;
    DmOpenRef           dbRef   = DmOpenDatabaseByTypeCreator(dbInfoP->type, dbInfoP->creatorId, dmModeReadOnly);

    //ȡԴ
    rscH = DmGet1Resource('taic', 1000);
    if (rscH == NULL)
    {
        dbInfoP->categoryId = 0;
    }
    else
    {
        rscP = (Char *)MemHandleLock(rscH);

        //򿪷
        cDbRef = DmOpenDatabaseByTypeCreator(CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
        if (DmGetLastErr() != errNone)  //ڷ
        {
            DmCreateDatabase(0, "Category List", APPLICATION_CREATOR, CATEGORY_LIST_DB_TYPE, false);
            cDbRef = DmOpenDatabaseByTypeCreator(CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
            if (cDbRef != NULL)
            {
                recordId = dmMaxRecordIndex;
                recordH = DmNewRecord(cDbRef, &recordId, sizeof(CategoryType));
                if (recordH != NULL)
                {
                    recordP = (CategoryType *)MemHandleLock(recordH);
                    DmSet(recordP, 0, sizeof(CategoryType), 0);
                    DmStrCopy(recordP, 2, STRING_UNFILED_CATEGORY);
                    MemHandleUnlock(recordH);
                    DmReleaseRecord(cDbRef, recordId, true);
                }
            }
        }

        //÷ڷı
        recordNums = DmNumRecordsInCategory(cDbRef, dmAllCategories);
        maxCategoryId = 0;
        dbInfoP->categoryId = 0;
        for (i = 0; i < recordNums; i ++)
        {
            recordH = DmQueryRecord(cDbRef, i);
            if (recordH == NULL)
                break;

            recordP = (CategoryType *)MemHandleLock(recordH);
            if (maxCategoryId < recordP->categoryId)
            {
                maxCategoryId = recordP->categoryId;
            }
            if (StrCompare(rscP, recordP->nameP) == 0)  //ҵ
            {
                dbInfoP->categoryId = recordP->categoryId;
                MemHandleUnlock(recordH);
                break;
            }
            MemHandleUnlock(recordH);
        }
        //ݳϢµķ
#if 0
        if (dbInfoP->categoryId == 0)   //ûжӦķ࣬½
        {
            maxCategoryId ++;
            recordId = dmMaxRecordIndex;
            recordH = DmNewRecord(cDbRef, &recordId, sizeof(CategoryType));
            recordP = (CategoryType *)MemHandleLock(recordH);
            DmSet(recordP, 0, sizeof(CategoryType), 0);
            DmWrite(recordP, 1, &maxCategoryId, 1);
            DmWrite(recordP, 2, rscP, StrLen(rscP));
            MemHandleUnlock(recordH);
            DmReleaseRecord(cDbRef, recordId, true);
            dbInfoP->categoryId = maxCategoryId;
        }
#endif
        //رշ
        if (cDbRef != NULL)
            DmCloseDatabase(cDbRef);

        MemHandleUnlock(rscH);

        DmReleaseResource(rscH);
    }

    if (dbRef != NULL)
        DmCloseDatabase(dbRef);
}

//ȡصķ
static Boolean isCategoryHide(UInt8 categoryId, GlobeType *globeP)
{
    UInt8       i;

    for (i = 0; i < globeP->hideCategoryCount; i ++)
    {
        if (globeP->hideCategoryP[i] == categoryId)
        {
            return true;
        }
    }

    return false;
}

//ȡбӦķ
static UInt8 getCategoryId(Int16 listIndex, UInt16 listCount, GlobeType *globeP)
{
    return globeP->categoryListMappingP[listIndex];
}

// С
static void loadTinyFont(GlobeType *globeP)
{
    FontType* tinyFontP;

    unloadTinyFont(globeP);
    globeP->tinyFontH = DmGetResource(TINY_FONT_RES_TYPE, TINY_FONT_RES_ID);
    if (globeP->tinyFontH != NULL)
    {
        tinyFontP = (FontType *)MemHandleLock(globeP->tinyFontH);
        if (FntDefineFont(TINY_FONT_ID, tinyFontP) != errNone)
        {
            ErrFatalDisplay("failed to load font resource!");
        }
    }
}

static void unloadTinyFont(GlobeType *globeP)
{
    if (globeP->tinyFontH != NULL)
    {
        MemHandleUnlock(globeP->tinyFontH);
        DmReleaseResource(globeP->tinyFontH);
        globeP->tinyFontH = NULL;
    }
}

#if 0
#pragma mark -----------------------------------------------------
#endif

//򿪳бݿ
static DmOpenRef openProgramDatabase()
{
    DmOpenRef       dbRef   = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);

    if (DmGetLastErr() != errNone)
    {
        DmCreateDatabase(0, "Program List", APPLICATION_CREATOR, PROGRAM_LIST_DB_TYPE, false);
        dbRef = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
    }

    return dbRef;
}

//ȡָĳڳбݿеλ
static Err getProgramId(DmOpenRef dbRef, UInt32 creatorId, UInt16 *recordId)
{
    MemHandle           recordH;
    ProgramListType     *recordP;
    UInt16              recordNum   = DmNumRecordsInCategory(dbRef, dmAllCategories);

    for ((*recordId) = 0; (*recordId) < recordNum; (*recordId) ++)
    {
        recordH = DmQueryRecord(dbRef, (*recordId));
        recordP = (ProgramListType *)MemHandleLock(recordH);
        if (recordP->creatorId == creatorId)
        {
            MemHandleUnlock(recordH);
            return errNone;
        }
        MemHandleUnlock(recordH);
    }

    return ERROR_PROGRAM_NOT_FOUND;
}

//ĳǷڳб
static Boolean isDbInDbList(DmOpenRef dbRef, UInt32 creatorId, Char *VFSNameP, UInt16 *recordIdP)
{
    UInt16              i;
    MemHandle           recordH;
    ProgramListType     *recordP;
    UInt16              recordNum   = DmNumRecordsInCategory(dbRef, dmAllCategories);

    for (i = 0; i < recordNum; i ++)
    {
        recordH = DmQueryRecord(dbRef, i);
        if (recordH != NULL)
        {
            recordP = (ProgramListType *)MemHandleLock(recordH);
            //ڴͿͬ
            if
            (
                (recordP->creatorId == creatorId)
                &&
                (
                    (
                        recordP->VFSNameP[0] == '\0'
                        &&
                        VFSNameP == NULL
                    )
                    ||
                    (
                        VFSNameP != NULL
                        &&
                        (StrNCompare(recordP->VFSNameP, VFSNameP, VFS_NAME_LENGTH) == 0)
                    )
                )
            )
            {
                MemHandleUnlock(recordH);
                if (recordIdP != NULL)
                {
                    (*recordIdP) = i;
                }
                return true;
            }
            MemHandleUnlock(recordH);
        }
    }

    return false;
}

static Char *strPreventChr(Char *stringP, Char chr)
{
    UInt16      stringLength    = StrLen(stringP);

    for (stringLength --; stringLength > 0; stringLength --)
    {
        if (stringP[stringLength] == chr)
        {
            return stringP + stringLength;
        }
    }

    return NULL;
}

//򻺴
static void buildProgramList(UInt8 mode, Boolean updateRAMPrograms, Boolean updateCardPrograms, GlobeType *globeP)
{
    UInt16              i;
    MemHandle           dbListH;
    SysDBListItemType   *dbListP;
    UInt16              dbListCount;
    ProgramListType     dbInfo;
    ProgramListType     *recordP;
    UInt16              attributes;
    MemHandle           recordH;
    UInt16              recordId;
    DmSearchStateType   dmSearchState;
    DmOpenRef           dbRef           = NULL;
    Char                nameP[256];

    switch (mode)
    {
        case BUILD_LIST_MODE_INIT:
            //ɾеķͳб
            if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, CATEGORY_LIST_DB_TYPE, APPLICATION_CREATOR, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
            {
                DmDeleteDatabase(dbInfo.cardNo, dbInfo.dbId);
            }
            if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
            {
                DmDeleteDatabase(dbInfo.cardNo, dbInfo.dbId);
            }

            //򿪳б
            dbRef = openProgramDatabase();

            if (dbRef != NULL)
            {
                //ȡڴеĳб
                if (SysCreateDataBaseList('appl', 0, &dbListCount, &dbListH, true))
                {
                    dbListP = (SysDBListItemType *)MemHandleLock(dbListH);
                    for (i = 0; i < dbListCount; i ++)
                    {
                        if (dbListP[i].creator != APPLICATION_CREATOR && dbListP[i].creator != sysFileCLauncher/* && dbListP[i].creator != PREF_CREATOR*/)
                        {
                            if (DmDatabaseInfo(dbListP[i].cardNo, dbListP[i].dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
                            {
                                if ((! (attributes & dmHdrAttrHidden)))
                                {
                                    dbInfo.cardNo = dbListP[i].cardNo;
                                    dbInfo.dbId = dbListP[i].dbID;
                                    dbInfo.type = dbListP[i].type;
                                    dbInfo.creatorId = dbListP[i].creator;

                                    //
                                    StrCopy(dbInfo.programNameP, dbListP[i].name);

                                    //ȡϢ
                                    getProgramCategory(&dbInfo);

                                    dbInfo.shortcut = chrNull;
                                    dbInfo.addOnInfo = 0;
                                    dbInfo.VFSNameP[0] = '\0';

                                    //
                                    recordId = dmMaxRecordIndex;
                                    recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
                                    if (recordH != NULL)
                                    {
                                        recordP = (ProgramListType *)MemHandleLock(recordH);
                                        if (recordP != NULL)
                                            DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
                                        MemHandleUnlock(recordH);
                                        DmReleaseRecord(dbRef, recordId, true);
                                    }
                                }
                            }
                        }
                    } // for loop
                    MemHandleUnlock(dbListH);
                    MemHandleFree(dbListH);
                }// SysCreateDataBaseList

                //ȡϵĳ
                {
                    FileInfoType    fileInfo;
                    FileRef         fileRef;
                    UInt32          type;
                    UInt32          creatorId;
                    UInt16          attributes;
                    UInt16          volRef          = 0;
                    UInt32          dirRef          = 0;
                    UInt32          volIterator     = vfsIteratorStart;
                    UInt32          dirIterator     = vfsIteratorStart;
                    Char            *fileExtName    = NULL;

                    //ȡָ
                    while (volIterator != vfsIteratorStop)
                    {
                        VFSVolumeEnumerate(&volRef, &volIterator);
                    }

                    if (volRef > 0)
                    {
                        if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeRead, &dirRef) == errNone) // /PALM/Launcher/
                        {
                            fileInfo.nameP = nameP;
                            fileInfo.nameBufLen = 255;

                            while (dirIterator != vfsIteratorStop)
                            {
                                if (VFSDirEntryEnumerate(dirRef, &dirIterator, &fileInfo) == errNone)
                                {
                                    fileExtName = strPreventChr(fileInfo.nameP, '.');   //ȡչ
                                    if (fileExtName != NULL)    //չ
                                    {
                                        if (StrNCaselessCompare(fileExtName, ".prc", 4) == 0)   //.prcΪһ
                                        {
                                            StrCopy(globeP->cacheP, "/PALM/Launcher/");
                                            StrCat(globeP->cacheP, fileInfo.nameP);
                                            if (VFSFileOpen(volRef, globeP->cacheP, vfsModeRead, &fileRef) == errNone)
                                            {
                                                //س
                                                if (VFSFileDBInfo(fileRef, dbInfo.programNameP, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
                                                {
                                                    if (type == 'appl' && (! (attributes & dmHdrAttrHidden)))
                                                    {
                                                        MemHandle tAIN_Handle;

                                                        dbInfo.cardNo = 0; //not available
                                                        dbInfo.dbId = 0;  //not available
                                                        dbInfo.type = type;
                                                        dbInfo.creatorId = creatorId;

                                                        //ȡϢ
                                                        getProgramCategory(&dbInfo);

                                                        dbInfo.shortcut = chrNull;
                                                        dbInfo.addOnInfo = 0;

                                                        //Դļ
                                                        if (VFSFileDBGetResource(fileRef, 'tAIN', 1000, &tAIN_Handle) == errNone)
                                                        {
                                                            char *appName = MemHandleLock(tAIN_Handle);
                                                            StrCopy(dbInfo.programNameP, appName);
                                                            MemHandleUnlock(tAIN_Handle);
                                                            MemHandleFree(tAIN_Handle);
                                                        }

                                                        //ļ
                                                        dbInfo.VFSNameP[0] = '\0';
                                                        StrNCat(dbInfo.VFSNameP, fileInfo.nameP, VFS_NAME_LENGTH);

                                                        //
                                                        recordId = dmMaxRecordIndex;
                                                        recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
                                                        if (recordH != NULL)
                                                        {
                                                            recordP = (ProgramListType *)MemHandleLock(recordH);
                                                            DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
                                                            MemHandleUnlock(recordH);
                                                            DmReleaseRecord(dbRef, recordId, true);
                                                        }
                                                    }
                                                }
                                                VFSFileClose(fileRef);
                                            }
                                        }
                                    }
                                }
                            }

                            VFSFileClose(dirRef);
                        }
                    }
                }

                //ǰĬ4
                if (getProgramId(dbRef, MEMO_CREATOR, &recordId) == errNone)    //±
                {
                    DmMoveRecord(dbRef, recordId, 0);
                }
                if (getProgramId(dbRef, CALCULATOR_CREATOR, &recordId) == errNone)  //ʼ
                {
                    DmMoveRecord(dbRef, recordId, 0);
                }
                if (getProgramId(dbRef, BLAZER_CREATOR, &recordId) == errNone)  //
                {
                    DmMoveRecord(dbRef, recordId, 0);
                }
                if (getProgramId(dbRef, CONTACTS_CREATOR, &recordId) == errNone)    //ͨѶ¼
                {
                    DmMoveRecord(dbRef, recordId, 0);
                }
                else if (getProgramId(dbRef, ADDRESS_CREATOR, &recordId) == errNone)
                {
                    DmMoveRecord(dbRef, recordId, 0);
                }

                DmCloseDatabase(dbRef);
            }
            break;

        case BUILD_LIST_MODE_REFRESH:
            //򿪳б
            dbRef = openProgramDatabase();

            if (dbRef != NULL)
            {
                //беļ¼Ƿ
                i = DmNumRecordsInCategory(dbRef, dmAllCategories);
                for (recordId = 0; recordId < i; recordId ++)
                {
                    recordH = DmQueryRecord(dbRef, recordId);
                    if (recordH != NULL)
                    {
                        recordP = (ProgramListType *)MemHandleLock(recordH);
                        //ڴ
                        if (recordP->VFSNameP[0] == '\0')
                        {
                            if (updateRAMPrograms)
                            {
                                if (DmGetNextDatabaseByTypeCreator(true, &dmSearchState, recordP->type, recordP->creatorId, true, &dbInfo.cardNo, &dbInfo.dbId) != dmErrCantFind)
                                {
                                    if (DmDatabaseInfo(dbInfo.cardNo, dbInfo.dbId, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
                                    {
                                        if ((attributes & dmHdrAttrHidden)) //أɾ
                                        {
                                            MemHandleUnlock(recordH);
                                            DmRemoveRecord(dbRef, recordId);
                                            i --;
                                            recordId --;
                                        }
                                        else
                                        {
                                            MemHandleUnlock(recordH);
                                        }
                                    }
                                    else    //޷ȡϢɾ
                                    {
                                        MemHandleUnlock(recordH);
                                        DmRemoveRecord(dbRef, recordId);
                                        i --;
                                        recordId --;
                                    }
                                }
                                else    //򲻴ڣɾ
                                {
                                    MemHandleUnlock(recordH);
                                    DmRemoveRecord(dbRef, recordId);
                                    i --;
                                    recordId --;
                                }
                            }
                            else
                            {
                                MemHandleUnlock(recordH);
                            }
                        }//ڴ
                        else if (updateCardPrograms)
                        {
                            UInt32          type;
                            UInt32          creatorId;
                            FileRef         fileRef;
                            UInt16          volRef          = 0;
                            UInt32          dirRef          = 0;
                            UInt32          volIterator     = vfsIteratorStart;

                            //ȡָ
                            while (volIterator != vfsIteratorStop)
                            {
                                VFSVolumeEnumerate(&volRef, &volIterator);
                            }

                            if (volRef > 0)
                            {
                                if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeRead, &dirRef) == errNone) // /PALM/Launcher/
                                {
                                    //·
                                    StrCopy(globeP->cacheP, "/PALM/Launcher/");
                                    StrCat(globeP->cacheP, recordP->VFSNameP);
                                    if (VFSFileOpen(volRef, globeP->cacheP, vfsModeRead, &fileRef) == errNone)
                                    {
                                        if (VFSFileDBInfo(fileRef, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
                                        {
                                            if (attributes & dmHdrAttrHidden)   //أɾ
                                            {
                                                MemHandleUnlock(recordH);
                                                DmRemoveRecord(dbRef, recordId);
                                                i --;
                                                recordId --;
                                            }
                                            else
                                            {
                                                MemHandleUnlock(recordH);
                                            }
                                        }   //ܻȡϢɾ
                                        else
                                        {
                                            MemHandleUnlock(recordH);
                                            DmRemoveRecord(dbRef, recordId);
                                            i --;
                                            recordId --;
                                        }
                                        VFSFileClose(fileRef);
                                    }
                                    else //򲻿ɾ
                                    {
                                        MemHandleUnlock(recordH);
                                        DmRemoveRecord(dbRef, recordId);
                                        i --;
                                        recordId --;
                                    }
                                    VFSFileClose(dirRef);
                                }
                                else //·ڣɾ
                                {
                                    MemHandleUnlock(recordH);
                                    DmRemoveRecord(dbRef, recordId);
                                    i --;
                                    recordId --;
                                }
                            }
                            else //ڣɾ
                            {
                                MemHandleUnlock(recordH);
                                DmRemoveRecord(dbRef, recordId);
                                i --;
                                recordId --;
                            }
                        }//ϳ
                        else
                        {
                            MemHandleUnlock(recordH);
                        }
                    }// recordH != NULL
                }// for loop

                if (updateRAMPrograms)
                {
                    //ȡڴеĳб
                    if (SysCreateDataBaseList('appl', 0, &dbListCount, &dbListH, true))
                    {
                        dbListP = (SysDBListItemType *)MemHandleLock(dbListH);

                        //Ƿµĳ
                        for (i = 0; i < dbListCount; i ++)
                        {
                            if (dbListP[i].creator != APPLICATION_CREATOR && dbListP[i].creator != sysFileCLauncher/* && dbListP[i].creator != PREF_CREATOR*/)
                            {
                                if (DmDatabaseInfo(dbListP[i].cardNo, dbListP[i].dbID, NULL, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == errNone)
                                {
                                    if ((! (attributes & dmHdrAttrHidden)))
                                    {
                                        if (! isDbInDbList(dbRef, dbListP[i].creator, NULL, NULL))
                                        {
                                            dbInfo.cardNo = dbListP[i].cardNo;
                                            dbInfo.dbId = dbListP[i].dbID;
                                            dbInfo.type = dbListP[i].type;
                                            dbInfo.creatorId = dbListP[i].creator;

                                            //ļ
                                            StrCopy(dbInfo.programNameP, dbListP[i].name);

                                            //ȡϢ
                                            getProgramCategory(&dbInfo);

                                            dbInfo.shortcut = chrNull;
                                            dbInfo.addOnInfo = 0;
                                            dbInfo.VFSNameP[0] = '\0';

                                            //
                                            recordId = dmMaxRecordIndex;
                                            recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
                                            recordP = (ProgramListType *)MemHandleLock(recordH);
                                            DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
                                            MemHandleUnlock(recordH);
                                            DmReleaseRecord(dbRef, recordId, true);
                                        }
                                    }
                                }
                            }
                        }
                        MemHandleUnlock(dbListH);
                        MemHandleFree(dbListH);
                    }
                }

                //鿨ϵ³
                if (updateCardPrograms)
                {
                    FileInfoType    fileInfo;
                    FileRef         fileRef;
                    UInt32          type;
                    UInt32          creatorId;
                    UInt16          attributes;
                    UInt16          volRef          = 0;
                    UInt32          dirRef          = 0;
                    UInt32          volIterator     = vfsIteratorStart;
                    UInt32          dirIterator     = vfsIteratorStart;
                    Char            *fileExtName    = NULL;

                    //ȡָ
                    while (volIterator != vfsIteratorStop)
                    {
                        VFSVolumeEnumerate(&volRef, &volIterator);
                    }

                    if (volRef > 0)
                    {
                        if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeRead, &dirRef) == errNone) // /PALM/Launcher/
                        {
                            fileInfo.nameP = nameP;
                            fileInfo.nameBufLen = 255;

                            while (dirIterator != vfsIteratorStop)
                            {
                                if (VFSDirEntryEnumerate(dirRef, &dirIterator, &fileInfo) == errNone)
                                {
                                    fileExtName = strPreventChr(fileInfo.nameP, '.');   //ȡչ
                                    if (fileExtName != NULL)    //չ
                                    {
                                        if (StrNCaselessCompare(fileExtName, ".prc", 4) == 0)   //.prcΪһ
                                        {
                                            StrCopy(globeP->cacheP, "/PALM/Launcher/");
                                            StrCat(globeP->cacheP, fileInfo.nameP);
                                            if (VFSFileOpen(volRef, globeP->cacheP, vfsModeRead, &fileRef) == errNone)
                                            {
                                                //س
                                                if (VFSFileDBInfo(fileRef, dbInfo.programNameP, &attributes, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &type, &creatorId, NULL) == errNone)
                                                {
                                                    if (type == 'appl' && (! (attributes & dmHdrAttrHidden)))
                                                    {
                                                        if (! isDbInDbList(dbRef, creatorId, fileInfo.nameP, NULL))
                                                        {
                                                            MemHandle   tAIN_Handle;

                                                            dbInfo.cardNo = 0; //not available
                                                            dbInfo.dbId = 0;  //not available
                                                            dbInfo.type = type;
                                                            dbInfo.creatorId = creatorId;

                                                            //ȡϢ
                                                            getProgramCategory(&dbInfo);

                                                            dbInfo.shortcut = chrNull;
                                                            dbInfo.addOnInfo = 0;

                                                            //Դļ
                                                            if (VFSFileDBGetResource(fileRef, 'tAIN', 1000, &tAIN_Handle) == errNone)
                                                            {
                                                                char *appName = MemHandleLock(tAIN_Handle);
                                                                StrCopy(dbInfo.programNameP, appName);
                                                                MemHandleUnlock(tAIN_Handle);
                                                                MemHandleFree(tAIN_Handle);
                                                            }

                                                            //ļ
                                                            dbInfo.VFSNameP[0] = '\0';
                                                            StrNCat(dbInfo.VFSNameP, fileInfo.nameP, VFS_NAME_LENGTH);

                                                            //
                                                            recordId = dmMaxRecordIndex;
                                                            recordH = DmNewRecord(dbRef, &recordId, sizeof(ProgramListType));
                                                            recordP = (ProgramListType *)MemHandleLock(recordH);
                                                            DmWrite(recordP, 0, &dbInfo, sizeof(ProgramListType));
                                                            MemHandleUnlock(recordH);
                                                            DmReleaseRecord(dbRef, recordId, true);
                                                        }
                                                    }
                                                }
                                                VFSFileClose(fileRef);
                                            }
                                        }
                                    }
                                }
                            }

                            VFSFileClose(dirRef);
                        }
                    }
                }   //ϵ³
                DmCloseDatabase(dbRef);
            }
            break;

    }
}

//ȡָΧĳϢ
static void getProgramListInCategory(UInt8 categoryId, GlobeType *globeP)
{
    UInt32              i;
    UInt16              endIndex;
    DmOpenRef           dbRef;
    MemHandle           recordH;
    ProgramListType     *recordP;

    //ճб
    freeGlobeProgramList(globeP);

    //ڴеĳ
    if (categoryId != CARD_CATEGORY)
    {
        //ݿ
        dbRef = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadOnly);

        if (dbRef != NULL)
        {
            //ȡΧ
            endIndex = DmNumRecordsInCategory(dbRef, dmAllCategories);

            //ڴ
            i = sizeof(ProgramListType) * endIndex;
            globeP->programListP = (ProgramListType *)MemPtrNew(i);
            MemSet(globeP->programListP, i, 0);

            //
            if (categoryId != ALL_CATEGORIES)
            {
                for (i = 0; i < endIndex; i ++)
                {
                    recordH = DmQueryRecord(dbRef, i);
                    recordP = (ProgramListType *)MemHandleLock(recordH);

                    //ûпԹϵĳ
                    if
                    (
                        recordP->categoryId == categoryId
                        &&
                        (
                            globeP->isVolumeMounted
                            ||
                            recordP->VFSNameP[0] == '\0'
                        )
                    )
                    {
                        globeP->programListP[globeP->programListCount] = (*recordP);
                        globeP->programListCount ++;
                    }
                    MemHandleUnlock(recordH);
                }
            }
            else
            {
                for (i = 0; i < endIndex; i ++)
                {
                    recordH = DmQueryRecord(dbRef, i);
                    recordP = (ProgramListType *)MemHandleLock(recordH);
                    if
                    (
                        !isCategoryHide(recordP->categoryId, globeP)
                        &&
                        (
                            globeP->isVolumeMounted
                            ||
                            recordP->VFSNameP[0] == '\0'
                        )
                    )
                    {
                        globeP->programListP[globeP->programListCount] = (*recordP);
                        globeP->programListCount ++;
                    }
                    MemHandleUnlock(recordH);
                }
            }

            //رݿ
            DmCloseDatabase(dbRef);
        }
    }
}

//ڸķм
static void searchProgramInCategory(Char *key, UInt8 categoryId, GlobeType *globeP)
{
    UInt32              i;
    UInt16              endIndex;
    UInt8               j;
    DmOpenRef           dbRef;
    MemHandle           recordH;
    ProgramListType     *recordP;
    UInt16              UInt32KeyLength;
    UInt16              UInt32NameLength;
    UInt16              contactSearchLibRef = 0;
    UInt32              *UInt32KeyP         = (UInt32 *)MemPtrNew(24);
    UInt32              *UInt32NameP        = (UInt32 *)MemPtrNew(24);

    //ѼؼתUInt32λװ
    convertCharsToBitPackage(key, UInt32KeyP, &UInt32KeyLength);

    //ʼճб
    freeGlobeProgramList(globeP);

    //ݿ
    dbRef = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadOnly);

    if (dbRef != NULL)
    {
        //ȡΧ
        endIndex = DmNumRecordsInCategory(dbRef, dmAllCategories);

        //ڴ
        i = sizeof(ProgramListType) * endIndex;
        globeP->programListP = (ProgramListType *)MemPtrNew(i);
        MemSet(globeP->programListP, i, 0);

        //
        for (i = 0; i < endIndex; i ++)
        {
            recordH = DmQueryRecord(dbRef, i);
            if (recordH != NULL)
            {
                recordP = (ProgramListType *)MemHandleLock(recordH);
                if
                (
                    (categoryId != ALL_CATEGORIES && recordP->categoryId == categoryId)
                    ||
                    (categoryId == ALL_CATEGORIES && !isCategoryHide(recordP->categoryId, globeP))
                )
                {
                    //תUInt32λװ
                    convertCharsToBitPackage(recordP->programNameP, UInt32NameP, &UInt32NameLength);

                    //Ƚ
                    j = 0;
                    if (UInt32KeyLength <= UInt32NameLength)
                    {
                        for (; j < UInt32KeyLength; j ++)
                        {
                            if ((UInt32KeyP[j] != UInt32NameP[j]))  //0Ҫж
                            {
                                break;
                            }
                        }
                    }

                    if (j == UInt32KeyLength)
                    {
                        globeP->programListP[globeP->programListCount] = (*recordP);
                        globeP->programListCount ++;
                    }
                }
                MemHandleUnlock(recordH);
            }
        }

        //رݿ
        DmCloseDatabase(dbRef);
    }

    MemPtrFree(UInt32NameP);
    MemPtrFree(UInt32KeyP);
}

//ȡơͼͿݼ
static void getProgramItemInfo(UInt8 deviceModel, ProgramListType *programP, ProgramItemInfoType *programInfoP, GlobeType *globeP)
{
    MemHandle       iconRscH;
    MemPtr          iconRscP;
    UInt16          bmpIconSize;

    //ʼֹγ
    programInfoP->bmpIconP = NULL;
    StrCopy(programInfoP->nameP, "-None-");

    if (programP->VFSNameP[0] == '\0')  //ڴ
    {
        DmOpenRef       dbRef           = DmOpenDatabaseByTypeCreator('appl', programP->creatorId, dmModeReadOnly);

        if (dbRef != NULL)
        {
            //
            if (programP->programNameP[0] != '\0')
            {
                StrCopy(programInfoP->nameP, programP->programNameP);
            }

            //ͼ
            iconRscH = DmGet1Resource('tAIB', 1000);
            if (iconRscH)
            {
                bmpIconSize = MemHandleSize(iconRscH);
                programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);
                if (programInfoP->bmpIconP != NULL)
                {
                    iconRscP = MemHandleLock(iconRscH);
                    MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
                    MemHandleUnlock(iconRscH);
                }
                DmReleaseResource(iconRscH);
                DmCloseDatabase(dbRef);
            }
            else    //ûͼ
            {
                DmCloseDatabase(dbRef);
                dbRef = DmOpenDatabaseByTypeCreator('appl', APPLICATION_CREATOR, dmModeReadOnly);
                if (dbRef != NULL)
                {
                    if (deviceModel != TREO_600)    //߷
                    {
                        iconRscH = DmGet1Resource('Tbmp', bmpNoIconLarge);
                    }
                    else    //ͷ
                    {
                        iconRscH = DmGet1Resource('Tbmp', bmpNoIconSmall);
                    }
                    if (iconRscH)
                    {
                        bmpIconSize = MemHandleSize(iconRscH);
                        programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);

                        if (programInfoP->bmpIconP != NULL)
                        {

                            iconRscP = MemHandleLock(iconRscH);
                            MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
                            MemHandleUnlock(iconRscH);
                        }
                        DmReleaseResource(iconRscH);
                    }
                    DmCloseDatabase(dbRef);
                }
            }
        }
    }
    else    //ڿ
    {
        FileRef         fileRef;
        UInt16          volRef          = 0;
        UInt32          dirRef          = 0;
        UInt32          volIterator     = vfsIteratorStart;

        //ȡָ
        while (volIterator != vfsIteratorStop)
        {
            VFSVolumeEnumerate(&volRef, &volIterator);
        }

        if (volRef > 0)
        {
            if (VFSFileOpen(volRef, "/PALM/Launcher", vfsModeRead, &dirRef) == errNone) // /PALM/Launcher/
            {
                //·
                StrCopy(globeP->cacheP, "/PALM/Launcher/");
                StrCat(globeP->cacheP, programP->VFSNameP);

                //򿪳ȡԴ
                if (VFSFileOpen(volRef, globeP->cacheP, vfsModeRead, &fileRef) == errNone)
                {
                    //ȡ
                    if (programP->programNameP[0] != '\0')
                    {
                        StrCopy(programInfoP->nameP, programP->programNameP);
                    }
                    else if (programP->VFSNameP[0] != '\0')
                    {
                        programInfoP->nameP[0] = '\0';
                        StrNCat(programInfoP->nameP, programP->VFSNameP, 32);
                    }

                    //ȡͼ
                    if (VFSFileDBGetResource(fileRef, 'tAIB', 1000, &iconRscH) == errNone)
                    {
                        bmpIconSize = MemHandleSize(iconRscH);
                        programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);
                        if (programInfoP->bmpIconP != NULL)
                        {
                            iconRscP = MemHandleLock(iconRscH);
                            MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
                            MemHandleUnlock(iconRscH);
                        }
                        MemHandleFree(iconRscH);
                    }
                    else    //ûͼ
                    {
                        DmOpenRef       dbRef   = DmOpenDatabaseByTypeCreator('appl', APPLICATION_CREATOR, dmModeReadOnly);

                        if (deviceModel != TREO_600)    //߷
                        {
                            iconRscH = DmGet1Resource('Tbmp', bmpNoIconLarge);
                        }
                        else    //ͷ
                        {
                            iconRscH = DmGet1Resource('Tbmp', bmpNoIconSmall);
                        }
                        bmpIconSize = MemHandleSize(iconRscH);
                        programInfoP->bmpIconP = (BitmapType *)MemPtrNew(bmpIconSize);

                        if (programInfoP->bmpIconP != NULL)
                        {
                            iconRscP = MemHandleLock(iconRscH);
                            MemMove(programInfoP->bmpIconP, iconRscP, bmpIconSize);
                            MemHandleUnlock(iconRscH);
                        }
                        DmReleaseResource(iconRscH);
                        DmCloseDatabase(dbRef);
                    }
                    VFSFileClose(fileRef);
                }
                else
                {
                    //ȡ
                    if (programP->programNameP[0] != '\0')
                    {
                        StrCopy(programInfoP->nameP, programP->programNameP);
                    }
                    else if (programP->VFSNameP[0] != '\0')
                    {
                        programInfoP->nameP[0] = '\0';
                        StrNCat(programInfoP->nameP, programP->VFSNameP, 32);
                    }
                }
                VFSFileClose(dirRef);
            }
        }
    }
}

#if 0
#pragma mark -
#pragma mark LAUNCHER FUNCTIONS
#pragma mark -
#endif

#if 0
#pragma mark == UI FUNCTIONS ==
#endif

//ȡǰ״̬
static UInt8 getToneStatus(Boolean *isVibrateON, GlobeType *globeP)
{
    SoundPreference     soundPrefs;
    UInt32              ringStatus;

    HsAttrGet(hsAttrRingSwitch, 0, &ringStatus);
    TonesLibGetSoundPrefs(globeP->tonesLibRef, soundPrefTypePhone, &soundPrefs);

    if (ringStatus != hsAttrRingSwitchMute)
    {
        (*isVibrateON) = soundPrefs.soundOnVibrate;
    }
    else
    {
        (*isVibrateON) = soundPrefs.soundOffVibrate;
        soundPrefs.soundOnVolume = 0;
    }


    return soundPrefs.soundOnVolume;
}

/*
㹫ʽ趨16Ϊ4
For CDMA, the number returned is the ABS(dBm). A value of 113 corresponds to
-113dBm.

According to GSM 07.07 Section 8.5, convert from dBm to rssi [0..31]:

0       -113 dBm or less
1       -111 dBm
2...30  -109... -53 dBm   [ (113 - s)/2 ]
31      -51 dBm or greater

To convert from rssi to bars [0..5]

0       -113 dBm or less
1       -111 dBm
2...4   -109... -53 dBm   [ ((rssi * 2)/15 + 1 ]
5       -51 dBm or greater


For GSM, I believe the number returned is rssi [0..31].

0       0..3
1..4    4..24   [ rssi/6 + 1 ]
5       25..n
*/

//ʾز˵ؼѡ
static void showHideMenuitem(GlobeType *globeP)
{
    //UInt16        libRef;
    //Err           errorCode;
    if (PhnLibModulePowered(globeP->hsPhoneLibRef) == phnPowerOn)
    {
        MenuHideItem(menuitemPhoneON);
        MenuShowItem(menuitemPhoneOFF);
    }
    else
    {
        MenuShowItem(menuitemPhoneON);
        MenuHideItem(menuitemPhoneOFF);
    }

    /*
    //ǷѾ
    if (SysLibFind("Net.lib", &libRef) == errNone)
    {
        NetLibOpenCount(libRef, &errorCode); //it is not working
        if (errorCode > 0)
        {
            MenuHideItem(menuitemConnectNetwork);
            MenuShowItem(menuitemDropNetwork);
        }
        else
        {
            MenuShowItem(menuitemConnectNetwork);
            MenuHideItem(menuitemDropNetwork);
        }
    }*/
}

//ָؼľ
static RectangleType *getFocusedObjectBounds(UInt16 objectId, GlobeType *globeP)
{
    switch (objectId)
    {
        case gdtCarrier:
            globeP->focusedObjectRectangle.topLeft.x = 2; globeP->focusedObjectRectangle.topLeft.y = 1;
            globeP->focusedObjectRectangle.extent.x = 80; globeP->focusedObjectRectangle.extent.y = 10;
            break;
        case gdtSound:
            globeP->focusedObjectRectangle.topLeft.x = 110; globeP->focusedObjectRectangle.topLeft.y = 1;
            globeP->focusedObjectRectangle.extent.x = 9; globeP->focusedObjectRectangle.extent.y = 10;
            break;
        case gdtBlueTooth:
            globeP->focusedObjectRectangle.topLeft.x = 120; globeP->focusedObjectRectangle.topLeft.y = 1;
            globeP->focusedObjectRectangle.extent.x = 9; globeP->focusedObjectRectangle.extent.y = 10;
            break;
        case gdtSignal:
            globeP->focusedObjectRectangle.topLeft.x = 131; globeP->focusedObjectRectangle.topLeft.y = 1;
            globeP->focusedObjectRectangle.extent.x = 14; globeP->focusedObjectRectangle.extent.y = 10;
            break;
        case gdtBattery:
            globeP->focusedObjectRectangle.topLeft.x = 150; globeP->focusedObjectRectangle.topLeft.y = 1;
            globeP->focusedObjectRectangle.extent.x = 8; globeP->focusedObjectRectangle.extent.y = 10;
            break;
        case gdtDatetime:
            globeP->focusedObjectRectangle.topLeft.x = 2; globeP->focusedObjectRectangle.topLeft.y = 15;
            globeP->focusedObjectRectangle.extent.x = 156; globeP->focusedObjectRectangle.extent.y = 11;
            break;
        case ptriCategory:
            FrmGetObjectBounds(globeP->frmPhoneUIP, FrmGetObjectIndex(globeP->frmPhoneUIP, ptriCategory), &globeP->focusedObjectRectangle);
            break;
/*
        case fldKeyword:
            globeP->focusedObjectRectangle.topLeft.x = 110; globeP->focusedObjectRectangle.topLeft.y = 148;
            globeP->focusedObjectRectangle.extent.x = 48; globeP->focusedObjectRectangle.extent.y = 11;
            break;
*/
    }

    return &globeP->focusedObjectRectangle;
}

//ȡʶ
static void getOperatorName(SettingType *settingP, GlobeType *globeP)
{
    if (PhnLibModulePowered(globeP->hsPhoneLibRef) == phnPowerOff)
    {
        StrCopy(globeP->operatorNameP, STRING_PHONE_OFF);
    }
    else
    {
        UInt32              E911Mode = true;

        HsAttrGet(hsAttrE911Mode, 0, &E911Mode);

        if ((Boolean)E911Mode == false)
        {
            if (settingP->useCustomCarrier)
            {
                StrCopy(globeP->operatorNameP, settingP->customCarrierP);
            }
            else
            {
                GSMRegistrationMode     gsmRegMode;
                PhnOperatorID           operatorID;
                Char                    *bufferP    = NULL;

                PhnLibCurrentOperator(globeP->hsPhoneLibRef, &operatorID, &bufferP, &gsmRegMode);
                if (bufferP)
                {
                    StrCopy(globeP->operatorNameP, bufferP);
                    MemPtrFree(bufferP);
                }
            }
        }
        else
        {
            StrCopy(globeP->operatorNameP, STRING_SOS_CALL_ONLY);
        }
    }
}

//ȡͼ
static void getBackgdBitmap(SettingType *settingP, GlobeType *globeP)
{
    UInt16          error;
    FileHand        fileH;
    void            *bmpBitsP   = NULL;

    //ͷžɵıͼ
    if (globeP->bmpStatusP != NULL)
    {
        BmpDelete((BitmapType *)globeP->bmpStatusP);
        globeP->bmpStatusP = NULL;
    }

    if (globeP->bmpBackgdP != NULL)
    {
        BmpDelete((BitmapType *)globeP->bmpBackgdP);
        globeP->bmpBackgdP = NULL;
    }

    if (FtrGet(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID, (UInt32 *)&bmpBitsP) == errNone)
    {
        FtrPtrFree(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID);
    }

    //ȡͼ
    fileH = FileOpen(0, WALLPAPER_CACHE_NAME, sysFileTFileStream, APPLICATION_CREATOR, fileModeReadOnly, &error);
    if (error == errNone)
    {
        UInt32      width, height;
        Int32       bmpBitsSize;
        Int32       bmpOffset;
        Int16       statusBarPos;


        FileTell(fileH, &bmpBitsSize, &error);
        bmpBitsSize -= 8;
        FileRead(fileH, &width, 4, 1, &error);
        FileRead(fileH, &height, 4, 1, &error);

        statusBarPos = globeP->statusBarRectangle.extent.y * 2 * height / 320;  //ٶ״̬0λÿʼ, double density
        height -= statusBarPos;
        bmpOffset = width * statusBarPos * 2;

        if (FtrPtrNew(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID, bmpBitsSize, &bmpBitsP) == errNone)
        {
            FileDmRead(fileH, bmpBitsP, 0, bmpBitsSize, 1, &error);

            if (error == errNone)
            {
                BitmapType  *bmpP;

                bmpP = BmpCreate(width, statusBarPos, 16, NULL, &error);
                if (error == errNone)
                {
                    globeP->bmpStatusP = BmpCreateBitmapV3(bmpP, kDensityDouble, bmpBitsP, NULL);
                    BmpDelete(bmpP);
                }

                bmpP = BmpCreate(width, height, 16, NULL, &error);
                if (error == errNone)
                {
                    globeP->bmpBackgdP = BmpCreateBitmapV3(bmpP, kDensityDouble, (void *)((UInt8 *)bmpBitsP + bmpOffset), NULL);
                    BmpDelete(bmpP);
                }
            }
            else
            {
                FtrPtrFree(APPLICATION_CREATOR, BITMAP_BITS_FEATURE_ID);
            }
        }

        FileClose(fileH);
    }
}

static void getSoundStatusBitmap(SettingType *settingP, GlobeType *globeP)
{
    UInt16          i;
    UInt16          bmpSize;
    MemHandle       bmpRscH;
    DmOpenRef       dbRef       = DmOpenDatabaseByTypeCreator('appl', APPLICATION_CREATOR, dmModeReadOnly);

    if (settingP->deviceModel != TREO_600)  //߷
    {
        for (i = 0; i < 3; i ++)
        {
            bmpRscH = DmGet1Resource('Tbmp', i + bmpSpeakerON);
            bmpSize = MemHandleSize(bmpRscH);

            globeP->bmpSoundStatusP[i] = (BitmapType *)MemPtrNew(bmpSize);
            MemMove(globeP->bmpSoundStatusP[i], MemHandleLock(bmpRscH), bmpSize);

            MemHandleUnlock(bmpRscH);
            DmReleaseResource(bmpRscH);
        }

        for (i = 0; i < 2; i++)
        {
            bmpRscH = DmGet1Resource('Tbmp', bmpPowerCharge + i);
            if (bmpRscH != NULL)
            {
                bmpSize = MemHandleSize(bmpRscH);

                globeP->bmpPowerStatusP[i] = (BitmapType *)MemPtrNew(bmpSize);
                MemMove(globeP->bmpPowerStatusP[i], MemHandleLock(bmpRscH), bmpSize);

                MemHandleUnlock(bmpRscH);
                DmReleaseResource(bmpRscH);
            }
        }

        for (i = 0; i < 3; i++)
        {
            bmpRscH = DmGet1Resource('Tbmp', bmpBlueToothOff + i);
            if (bmpRscH != NULL)
            {
                bmpSize = MemHandleSize(bmpRscH);

                globeP->bmpBlueToothStatusP[i] = (BitmapType *)MemPtrNew(bmpSize);
                MemMove(globeP->bmpBlueToothStatusP[i], MemHandleLock(bmpRscH), bmpSize);

                MemHandleUnlock(bmpRscH);
                DmReleaseResource(bmpRscH);
            }
        }

        bmpRscH = DmGet1Resource('Tbmp', bmpSearch);
        if (bmpRscH != NULL)
        {
            bmpSize = MemHandleSize(bmpRscH);

            globeP->bmpSearchP = (BitmapType *)MemPtrNew(bmpSize);
            MemMove(globeP->bmpSearchP, MemHandleLock(bmpRscH), bmpSize);

            MemHandleUnlock(bmpRscH);
            DmReleaseResource(bmpRscH);
        }
    }

    DmCloseDatabase(dbRef);
}

static void setRectangleData(GlobeType *globeP)
{
    globeP->screenRectangle.topLeft.x = 0;
    globeP->screenRectangle.topLeft.y = 0;
    globeP->screenRectangle.extent.x = 160;
    globeP->screenRectangle.extent.y = 160;

    globeP->backgdRectangle.topLeft.x = 0;
    globeP->backgdRectangle.topLeft.y = 13;
    globeP->backgdRectangle.extent.x = 160;
    globeP->backgdRectangle.extent.y = 160;

    globeP->statusBarRectangle.topLeft.x = 0;
    globeP->statusBarRectangle.topLeft.y = 0;
    globeP->statusBarRectangle.extent.x = 160;
    globeP->statusBarRectangle.extent.y = 13;

    globeP->datetimeBarRectangle.topLeft.x = 0;
    globeP->datetimeBarRectangle.topLeft.y = 13;
    globeP->datetimeBarRectangle.extent.x = 160;
    globeP->datetimeBarRectangle.extent.y = 15;

    globeP->launchBarRectangle.topLeft.x = 0;
    globeP->launchBarRectangle.topLeft.y = 115;
    globeP->launchBarRectangle.extent.x = 160;
    globeP->launchBarRectangle.extent.y = 45;

    globeP->launchViewRectangle.topLeft.x = 0;
    globeP->launchViewRectangle.topLeft.y = 13;
    globeP->launchViewRectangle.extent.x = 160;
    globeP->launchViewRectangle.extent.y = 147;

    globeP->launchViewTitleBar.topLeft.x = 0;
    globeP->launchViewTitleBar.topLeft.y = 15;
    globeP->launchViewTitleBar.extent.x = 160;
    globeP->launchViewTitleBar.extent.y = 12;

    globeP->launchViewStatusBar.topLeft.x = 0;
    globeP->launchViewStatusBar.topLeft.y = 146;
    globeP->launchViewStatusBar.extent.x = 160;
    globeP->launchViewStatusBar.extent.y = 14;

    /*globeP->positionRectangle[0].topLeft.x = 0;
    globeP->positionRectangle[0].extent.x = 40;

    globeP->positionRectangle[1].topLeft.x = 40;
    globeP->positionRectangle[1].extent.x = 40;

    globeP->positionRectangle[2].topLeft.x = 80;
    globeP->positionRectangle[2].extent.x = 40;

    globeP->positionRectangle[3].topLeft.x = 120;
    globeP->positionRectangle[3].extent.x = 40;*/
}

#if 0
#pragma mark -----------------------------------------------------
#endif

//
static void drawIconFocusedRing(RectangleType *rectangleP, UInt8 style, SettingType *settingP, GlobeType *globeP)
{
    Int16       i, n;
    Coord       x;
    Coord       y;

    if (settingP->deviceModel != TREO_600)
    {
        RGBColorType    oldForeColor;

        WinSetForeColorRGB(&settingP->formFillColor, &oldForeColor);
        switch (style)
        {
            case FOCUS_RING_STYLE_ROUND_RECTANGLE:
                HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleRound, false);
                break;
            case FOCUS_RING_STYLE_SHARP_RECTANGLE:
                HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleSquare, false);
                break;
            case FOCUS_RING_STYLE_HORIZONTAL_BARS:
                HsNavDrawFocusRing(globeP->frmPhoneUIP, gdtIcons, hsNavFocusRingNoExtraInfo, rectangleP, hsNavFocusRingStyleHorizontalBars, false);
                break;
            case FOCUS_RING_STYLE_BOTTOM_BAR:
                WinSetCoordinateSystem(kCoordinatesDouble);

                x = (rectangleP->topLeft.x - 3) * 2 + 3;
                y = (rectangleP->topLeft.y + rectangleP->extent.y - 1) * 2;
                n = (rectangleP->topLeft.x + rectangleP->extent.x + 3) * 2 - 3;

                for (i = 5; i >= 3; i --)
                {
                    WinSetForeColorRGB(&settingP->focusRingColor[i], NULL);
                    WinPaintLine(x, y, n, y);
                    y ++;
                    x --;
                    n ++;
                }
                x ++;
                n --;
                for (i = 2; i >= 0; i --)
                {
                    WinSetForeColorRGB(&settingP->focusRingColor[i], NULL);
                    WinPaintLine(x, y, n, y);
                    y ++;
                    x ++;
                    n --;
                }

                WinSetCoordinateSystem(kCoordinatesStandard);
                break;
            case FOCUS_RING_STYLE_ANGLE:
                WinSetCoordinateSystem(kCoordinatesDouble);

                {
                    WinLineType     l[4][2];

                    l[0][0].x1 = (rectangleP->topLeft.x - 3) * 2;
                    l[0][0].y1 = (rectangleP->topLeft.y - 3) * 2;
                    l[0][0].x2 = l[0][0].x1;
                    l[0][0].y2 = l[0][0].y1 + 18;
                    l[0][1].x1 = l[0][0].x1;
                    l[0][1].y1 = l[0][0].y1;
                    l[0][1].x2 = l[0][1].x1 + 12;
                    l[0][1].y2 = l[0][1].y1;

                    l[1][0].x1 = (rectangleP->topLeft.x + rectangleP->extent.x + 3) * 2;
                    l[1][0].y1 = l[0][0].y1;
                    l[1][0].x2 = l[1][0].x1;
                    l[1][0].y2 = l[1][0].y1 + 18;
                    l[1][1].x1 = l[1][0].x1;
                    l[1][1].y1 = l[1][0].y1;
                    l[1][1].x2 = l[1][1].x1 - 12;
                    l[1][1].y2 = l[1][1].y1;

                    l[2][0].x1 = l[0][0].x1;
                    l[2][0].y1 = (rectangleP->topLeft.y + rectangleP->extent.y + 2) * 2;
                    l[2][0].x2 = l[2][0].x1;
                    l[2][0].y2 = l[2][0].y1 - 18;
                    l[2][1].x1 = l[2][0].x1;
                    l[2][1].y1 = l[2][0].y1;
                    l[2][1].x2 = l[2][1].x1 + 12;
                    l[2][1].y2 = l[2][1].y1;

                    l[3][0].x1 = l[1][0].x1;
                    l[3][0].y1 = l[2][0].y1;
                    l[3][0].x2 = l[3][0].x1;
                    l[3][0].y2 = l[3][0].y1 - 18;
                    l[3][1].x1 = l[3][0].x1;
                    l[3][1].y1 = l[3][0].y1;
                    l[3][1].x2 = l[3][1].x1 - 12;
                    l[3][1].y2 = l[3][1].y1;

                    for (i = 5; i >= 0; i --)
                    {
                        WinSetForeColorRGB(&settingP->focusRingColor[i], NULL);

                        for (n = 0; n < 4; n ++)
                        {
                            WinPaintLines(2, l[n]);
                        }

                        l[0][0].x1 ++;
                        l[0][0].y1 ++;
                        l[0][0].x2 ++;
                        l[0][1].x1 ++;
                        l[0][1].y1 ++;
                        l[0][1].y2 ++;

                        l[1][0].x1 --;
                        l[1][0].y1 ++;
                        l[1][0].x2 --;
                        l[1][1].x1 --;
                        l[1][1].y1 ++;
                        l[1][1].y2 ++;

                        l[2][0].x1 ++;
                        l[2][0].y1 --;
                        l[2][0].x2 ++;
                        l[2][1].x1 ++;
                        l[2][1].y1 --;
                        l[2][1].y2 --;

                        l[3][0].x1 --;
                        l[3][0].y1 --;
                        l[3][0].x2 --;
                        l[3][1].x1 --;
                        l[3][1].y1 --;
                        l[3][1].y2 --;
                    }
                }

                WinSetCoordinateSystem(kCoordinatesStandard);
                break;
            case FOCUS_RING_STYLE_ROUND_FRAME:
                rectangleP->topLeft.x --;
                rectangleP->topLeft.y --;
                rectangleP->extent.x += 2;
                rectangleP->extent.y += 2;
                WinSetForeColorRGB(&settingP->focusRingColor[0], NULL);
                WinPaintRectangleFrame(roundFrame, rectangleP);
                break;
            case FOCUS_RING_STYLE_FRMAE_WITH_MASK:
                rectangleP->topLeft.x --;
                rectangleP->topLeft.y --;
                rectangleP->extent.x += 2;
                rectangleP->extent.y += 2;
                WinSetForeColorRGB(&settingP->focusRingColor[3], NULL);
                WinPaintRectangle (rectangleP, 4);
                WinSetForeColorRGB(&settingP->focusRingColor[0], NULL);
                WinPaintRectangleFrame(roundFrame, rectangleP);
                break;
        }
        WinSetForeColorRGB(&oldForeColor, NULL);
    }
}

//
static void drawPhoneUIBackground(SettingType *settingP, GlobeType *globeP)
{
    if (settingP->shouldShowBackgd && globeP->bmpBackgdP)   //ʾͼ
    {
        //汳ͼ
        WinDrawBitmap((BitmapType *)globeP->bmpBackgdP, 0, globeP->backgdRectangle.topLeft.y);
    }
    else
    {
        RGBColorType    oldBackColor;

        //ʾɫ
        WinSetBackColorRGB(&settingP->formFillColor, &oldBackColor);
        WinEraseRectangle(&globeP->backgdRectangle, 0);
        WinSetBackColorRGB(&oldBackColor, NULL);
    }
}

static void drawPhoneUISignalBattery(SettingType *settingP, GlobeType *globeP)
{
    UInt16              i;
    RGBColorType        blackColor = {0, 0, 0, 0};
    UInt16              barLevel;
    Boolean             batteryPlugin = false;
    UInt16              batteryVolt;
    UInt16              batteryWarnThreshold;
    UInt8               batteryPercent = 0;
    RGBColorType        oldForeColor;
    RGBColorType        oldTextColor;
    Char                bufferP[16];
    FontID              oldFont = FntSetFont(TINY_FONT_ID);


    //ָֻ֧߷ֱĻ
    if (!globeP->isHighDensitySupported)
        return;

    WinSetForeColorRGB(&settingP->formFillColor, &oldForeColor);
    WinSetTextColorRGB(&settingP->statusDigitalColor, &oldTextColor);

    WinSetCoordinateSystem(kCoordinatesDouble);

    if (PhnLibModulePowered(globeP->hsPhoneLibRef) == phnPowerOn)
    {
        UInt32          phoneType = 0;
        UInt16          phoneSignalQuality = 0;
        UInt8           phoneRssi;
        RGBColorType    strengthColor;

        PhnLibSignalQuality(globeP->hsPhoneLibRef, &phoneSignalQuality);

        if (phoneSignalQuality <= 13)
            phoneRssi = (UInt8)(phoneSignalQuality / 4);
        else
            phoneRssi = 4;

        WinSetForeColorRGB(&settingP->statusDigitalColor, NULL);
        WinPaintLines(SIGNAL_ONLY_LENGTH, signalOnly);

        switch(phoneRssi)
        {
            case 1:
            strengthColor.index = 0;
            strengthColor.r = 241;
            strengthColor.g = 0;
            strengthColor.b = 0;
            break;

            case 2:
            strengthColor.index = 0;
            strengthColor.r = 255;
            strengthColor.g = 156;
            strengthColor.b = 0;
            break;

            case 3:
            strengthColor.index = 0;
            strengthColor.r = 0;
            strengthColor.g = 156;
            strengthColor.b = 0;
            break;

            case 4:
            strengthColor.index = 0;
            strengthColor.r = 0;
            strengthColor.g = 0;
            strengthColor.b = 201;
            break;

            default:
            strengthColor.index = 0;
            strengthColor.r = 0;
            strengthColor.g = 0;
            strengthColor.b = 0;
            break;
        }

        WinSetForeColorRGB(&strengthColor, NULL);
        for (i = 1; i <= phoneRssi; i++)
        {
            WinPaintLines(3, signalLevel[i-1]);
        }

        //źǿʾ
        StrIToA(bufferP, phoneSignalQuality);
        if (phoneSignalQuality >= 10)
            WinPaintChars(bufferP, StrLen(bufferP), 290, 1);
        else
            WinPaintChars(bufferP, StrLen(bufferP), 296, 1);

        //detect the device type
        HsAttrGet(hsAttrPhoneType, 0, &phoneType);

        if (phoneType == hsAttrPhoneTypeGSM)
        {
            UInt16          libRef = 0;
            Err             errorCode;
            Boolean         networkUp = false;
            NetMasterPBType pb;

            errorCode = SysLibFind("Net.lib", &libRef);
            if (errorCode == errNone && libRef != 0)
            {
                pb.param.interfaceInfo.index = 0;
                errorCode = NetLibMaster(libRef, netMasterInterfaceInfo, &pb, 0);
                if
                (
                    errorCode == errNone
                    &&
                    pb.param.interfaceInfo.driverUp
                    &&
                    pb.param.interfaceInfo.ifUp
                )
                {
                    RGBColorType greenColor = {0, 0, 255, 0};

                    WinSetForeColorRGB(&greenColor, NULL);
                    WinPaintLines(SIGNAL_CONNECTION_LENGTH, signalConnection);

                    networkUp = true;
                }
            }

            if (!networkUp && PhnLibGPRSAttached(globeP->hsPhoneLibRef))
            {
                WinSetForeColorRGB(&blackColor, NULL);
                WinPaintLines(SIGNAL_DATA_LENGTH, signalData);
            }
        }
    }
    else //ɫźű־
    {
        RGBColorType color = {0, 222, 22, 8};

        WinSetForeColorRGB(&color, NULL);
        WinPaintLines(SIGNAL_ONLY_LENGTH, signalOnly);
    }

    //
    WinSetForeColorRGB(&blackColor, NULL);
    WinPaintLines(BATTERY_SHAPE_LENGTH, batteryShape);

    batteryVolt = SysBatteryInfo(false, &batteryWarnThreshold, NULL, NULL, NULL, &batteryPlugin, &batteryPercent);

    //ʾ
    StrIToA(bufferP, batteryPercent);
    if (batteryPercent >= 100)
        WinPaintChars(bufferP, StrLen(bufferP), 285, 13);
    else if (batteryPercent >= 10)
        WinPaintChars(bufferP, StrLen(bufferP), 290, 13);
    else
        WinPaintChars(bufferP, StrLen(bufferP), 296, 13);


    barLevel = 22 - batteryPercent * (22 - 4) / 100;

    //ɫ
    if (batteryVolt <= batteryWarnThreshold || batteryPercent <= 20)
    {
        RGBColorType redColor = {0, 204, 0, 51};
        UInt8 i;

        WinSetForeColorRGB(&redColor, NULL);
        for (i = 0; i < 9; i++)
        {
            WinPaintLine(306 + i, 22, 306 + i, barLevel);
        }
    }
    else    //ɫʾ
    {
        RGBColorType    lineColor;

        lineColor.index = 0;

        lineColor.r = 255;
        lineColor.g = 255;
        lineColor.b = 255;

        WinSetForeColorRGB(&lineColor, NULL);
        for (i = 306; i < 309; i++)
            WinPaintLine(i, 4, i, barLevel);
        for (i = 312; i < 315; i++)
            WinPaintLine(i, 4, i, barLevel);
        if (batteryPercent != 100)
        {
            for (i = 309; i < 312; i++)
                WinPaintLine(i, 2, i, barLevel);
        }

        lineColor.r = 0;
        lineColor.g = 0;
        lineColor.b = 100;
        WinSetForeColorRGB(&lineColor, NULL);
        WinPaintLine(306, 22, 306, barLevel);
        WinPaintLine(314, 22, 314, barLevel);

        lineColor.r = 0;
        lineColor.g = 48;
        lineColor.b = 201;
        WinSetForeColorRGB(&lineColor, NULL);
        WinPaintLine(307, 22, 307, barLevel);
        WinPaintLine(313, 22, 313, barLevel);

        lineColor.r = 0;
        lineColor.g = 48;
        lineColor.b = 255;
        WinSetForeColorRGB(&lineColor, NULL);
        WinPaintLine(308, 22, 308, barLevel);
        WinPaintLine(312, 22, 312, barLevel);

        lineColor.r = 54;
        lineColor.g = 154;
        lineColor.b = 255;
        WinSetForeColorRGB(&lineColor, NULL);
        if (batteryPercent != 100)
        {
            WinPaintLine(309, 22, 309, barLevel);
            WinPaintLine(311, 22, 311, barLevel);
        }
        else
        {
            WinPaintLine(309, 22, 309, 2);
            WinPaintLine(311, 22, 311, 2);
        }
        lineColor.r = 100;
        lineColor.g = 207;
        lineColor.b = 255;
        WinSetForeColorRGB(&lineColor, NULL);
        if (batteryPercent != 100)
            WinPaintLine(310, 22, 310, barLevel);
        else
            WinPaintLine(310, 22, 310, 2);
    }

    //־
    if (batteryPlugin)
    {
        if (batteryPercent != 100 && globeP->bmpPowerStatusP[0] != NULL)
            WinPaintBitmap(globeP->bmpPowerStatusP[0], 303, 6);
        else if (globeP->bmpPowerStatusP[1] != NULL)
            WinPaintBitmap(globeP->bmpPowerStatusP[1], 303, 6);
    }

    WinSetForeColorRGB(&oldForeColor, NULL);
    WinSetTextColorRGB(&oldTextColor, NULL);
    FntSetFont(oldFont);

    WinSetCoordinateSystem(kCoordinatesStandard);
}

//״̬
static void drawPhoneUIStatusBar(SettingType *settingP, GlobeType *globeP)
{
    if (! globeP->isMenuActived)
    {
        UInt8           phoneSoundVolume;
        Boolean         isVibrateON;
        UInt16          attnCount;
        FontID          oldFont;
        RGBColorType    oldTextColor;

        globeP->ticksUpdateStatusBar = TimGetTicks();

        //״̬
        if (settingP->shouldShowBackgd && globeP->bmpStatusP)   //ʾͼ
        {
            //汳ͼ
            WinDrawBitmap((BitmapType *)globeP->bmpStatusP, 0, 0);
        }
        else
        {
            WinSetBackColorRGB(&settingP->topStatusFillColor, NULL);
            WinEraseRectangle(&globeP->statusBarRectangle, 0);
        }

        //־
        WinSetTextColorRGB(&settingP->textColor, &oldTextColor);
        oldFont = FntSetFont(boldFont);

        attnCount = AttnGetCounts(0, 0, NULL, NULL);
        if(attnCount == 0)
        {
            paintCharsWithOutline(globeP->operatorNameP, StrLen(globeP->operatorNameP), 1, 1, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }
        else
        {
            paintCharsWithOutline(STRING_ALERT_MANAGER, StrLen(STRING_ALERT_MANAGER), 1, 1, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }

        WinSetTextColorRGB(&oldTextColor, NULL);
        FntSetFont(oldFont);

        //Gadget
//      PmSysGadgetStatusGadgetsUpdate(globeP->pmGadgetLibRef);

        if (globeP->isHighDensitySupported)
        {
            UInt16              size;

            //״̬
            phoneSoundVolume = getToneStatus(&isVibrateON, globeP);

            if (phoneSoundVolume == 0)  //
            {
                WinPaintBitmap(globeP->bmpSoundStatusP[1], 108, 1);
            }
            else if (phoneSoundVolume <= 7) //
            {
                RGBColorType    oldTextColor;
                FontID          oldFont = FntSetFont(TINY_FONT_ID);

                WinPaintBitmap(globeP->bmpSoundStatusP[0], 108, 1);

                WinSetTextColorRGB(&settingP->statusDigitalColor, &oldTextColor);
                StrIToA(globeP->cacheP, phoneSoundVolume);
                WinPaintChars(globeP->cacheP, 1, 115, 7);

                WinSetTextColorRGB(&oldTextColor, NULL);
                FntSetFont(oldFont);
            }
            if (isVibrateON)    //𶯿
            {
                WinPaintBitmap(globeP->bmpSoundStatusP[2], 115, 1);
            }

            //״̬
            //ʵṹ BluetoothPanelPrefsType
            size = 0;
            if (PrefGetAppPreferences(sysFileCBluetoothPanel, 1, NULL, &size, true) != noPreferenceFound)
            {
                UInt8 *ptr = MemPtrNew(size);
                if (ptr != NULL)
                {
                    PrefGetAppPreferences(sysFileCBluetoothPanel, 1, ptr, &size, true);
                    if (ptr[0])
                    {
                        if (globeP->bmpBlueToothStatusP[1] != NULL)
                            WinPaintBitmap(globeP->bmpBlueToothStatusP[1], 121, 1);
                    }
                    else
                    {
                        if (globeP->bmpBlueToothStatusP[0] != NULL)
                            WinPaintBitmap(globeP->bmpBlueToothStatusP[0], 121, 1);
                    }
                    MemPtrFree(ptr);
                }
            }
        }

        //źǿȺ͵ʾ
        drawPhoneUISignalBattery(settingP, globeP);
    }
}

//
static void drawPhoneUICalendar(DateTimeType *nowP, SettingType *settingP, GlobeType *globeP)
{
    RGBColorType    colorGray       = {0, 204, 204, 204};
    RGBColorType    colorRed        = {0, 255, 0, 0};
    RGBColorType    colorBlue       = {0, 0, 0, 255};
    RGBColorType    oldTextColor;
    const Char      *weekTitle[7]   = {STRING_WEEK_TITLE};
    UInt8           daysInMonth     = DaysInMonth(nowP->month, nowP->year);
    UInt8           day, i, dow;
    UInt8           startDayOfWeek  = PrefGetPreference(prefWeekStartDay);
    Coord           x, y;
    UInt16          stringLength;
    Char            dayBufferP[maxStrIToALen];
    FontID          oldFont;

    oldFont = FntSetFont(stdFont);
    WinSetTextColorRGB(&settingP->textColor, &oldTextColor);

    //
    for (i = 0; i < 7; i ++)
    {
        dow = (startDayOfWeek + i) % 7;

        x = 3 + 22 * i;
        x += 11 - FntCharsWidth(weekTitle[dow], WEEK_TITLE_LENGTH) / 2;
        if (dow == 0 || dow == 6)
        {
            WinSetTextColorRGB(&colorRed, NULL);
            paintCharsWithOutline(weekTitle[dow], WEEK_TITLE_LENGTH, x, 29, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
            WinSetTextColorRGB(&settingP->textColor, NULL);
        }
        else
        {
            paintCharsWithOutline(weekTitle[dow], WEEK_TITLE_LENGTH, x, 29, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }
    }

    //
    dow = DayOfWeek(nowP->month, 1, nowP->year);
    i = (dow - startDayOfWeek + 7) % 7;

    y = 40;
    for (day = 1; day <= daysInMonth; day ++)
    {
        x = 22 * i + 3;
        StrIToA(dayBufferP, day);
        stringLength = StrLen(dayBufferP);

        dow = (startDayOfWeek + i) % 7;
        if (dow == 0 || dow == 6)
        {
            WinSetTextColorRGB(&colorRed, NULL);
        }

        if (day == nowP->day)
        {
            WinSetTextColorRGB(&colorBlue, NULL);
            FntSetFont(boldFont);
        }

        x += 11 - FntCharsWidth(dayBufferP, stringLength) / 2;
        paintCharsWithOutline(dayBufferP, stringLength, x, y, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);

        if (day == nowP->day)
        {
            FntSetFont(stdFont);
        }
        if (day == nowP->day || dow == 0 || dow == 6)
        {
            WinSetTextColorRGB(&settingP->textColor, NULL);
        }

        i++;
        if (i > 6)
        {
            i = 0;
            y += 11;
        }
    }

    FntSetFont(oldFont);
    WinSetTextColorRGB(&oldTextColor, NULL);
}

//
static void drawPhoneUIContent(SettingType *settingP, GlobeType *globeP)
{
    UInt16          stringLength;
    UInt32          i;
    UInt32          result;
    DateType        refDate = {2000, 2, 5};
    DateType        today;
    DateTimeType    now;
    LunarDateType   lunarDate;
    RGBColorType    oldForeColor;
    RGBColorType    oldTextColor;
    FontID          oldFont;

    //ȡǰʱ
    TimSecondsToDateTime(TimGetSeconds(), &now);

    if (globeP->isInPhoneView)
    {
        if (settingP->shouldShowDateTime)   //ʾʱ
        {
            //ȡʱ
            DateTemplateToAscii(STRING_DATE_FORMAT, now.month, now.day, now.year, globeP->dateStringP, 64);

            //ũ
            today.year  = now.year;
            today.month = now.month;
            today.day   = now.day;

            if(solar2Lunar(&lunarDate, DateToDays(today) - DateToDays(refDate)))
            {
                Char textInfo[16];

                StrPrintF(textInfo," %s%s",lunarDate.month, lunarDate.day);
                StrCat(globeP->dateStringP, textInfo);
            }
            else
            {
                StrCat(globeP->dateStringP, "ǿ");
            }

            StrCat(globeP->dateStringP, " ");
            switch (now.weekDay)
            {
                case 0:
                    StrCat(globeP->dateStringP, "");
                    break;
                case 1:
                    StrCat(globeP->dateStringP, "һ");
                    break;
                case 2:
                    StrCat(globeP->dateStringP, "");
                    break;
                case 3:
                    StrCat(globeP->dateStringP, "");
                    break;
                case 4:
                    StrCat(globeP->dateStringP, "");
                    break;
                case 5:
                    StrCat(globeP->dateStringP, "");
                    break;
                case 6:
                    StrCat(globeP->dateStringP, "");
                    break;
            }

            TimeToAscii(now.hour, now.minute, tfColon24h, globeP->timeStringP);
            stringLength = StrLen(globeP->timeStringP);

            oldFont = FntSetFont(boldFont);
            WinSetForeColorRGB(&settingP->formFrameColor, &oldForeColor);

            if (settingP->shouldAddMask)    //ֲ
            {
                WinPaintRectangle(&globeP->datetimeBarRectangle, 0);

                WinSetTextColorRGB(&settingP->textOnMaskColor, &oldTextColor);
                WinPaintChars(globeP->dateStringP, StrLen(globeP->dateStringP), 3, 15);
                WinPaintChars(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15);
            }
            else
            {
                if (settingP->shouldAddLine)
                {
                    WinPaintLine(0, 13, 159, 13);
                    WinPaintLine(0, 27, 159, 27);
                }

                WinSetTextColorRGB(&settingP->textColor, &oldTextColor);
                paintCharsWithOutline(globeP->dateStringP, StrLen(globeP->dateStringP), 3, 15, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
                paintCharsWithOutline(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
            }

            if (settingP->shouldShowCalendar)   //ʾ
            {
                drawPhoneUICalendar(&now, settingP, globeP);
            }
        }
        else if (settingP->shouldAddLine)
        {
            WinSetForeColorRGB(&settingP->formFrameColor, &oldForeColor);
            WinPaintLine(0, 13, 159, 13);
        }

        //ʾ
        if (!settingP->shouldShowCalendar)
        {
            for (i = 0; i < globeP->pluginListCount; i ++)
            {
                //NeoʽĲ
                if (globeP->pluginListP[i].creatorId == APPLICATION_CREATOR)
                {
                    globeP->pluginParameter.actionType = ACTION_TYPE_DRAW;
                    globeP->pluginParameter.pluginRectangle = globeP->pluginListP[i].pluginRectangle;
                    globeP->pluginParameter.pluginDataP = globeP->pluginListP[i].pluginDataP;
                    SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, LAUNCH_CODE_PLUGIN_ACTION, &globeP->pluginParameter, &result);
                    globeP->pluginListP[i].pluginDataP = globeP->pluginParameter.pluginDataP;
                    globeP->pluginListP[i].pluginRectangle = globeP->pluginParameter.pluginRectangle;
                }
                else if (globeP->pluginListP[i].type == ZL_PLUGIN_LIST_DB_TYPE)
                {
                    ZLPluginData *dataP = (ZLPluginData *)(globeP->pluginListP[i].pluginDataP);

                    if (dataP != NULL && dataP->isPluginOpened && dataP->cmdBlockTypeEx.cmdBlockType.dbP != NULL)
                    {
                        UInt32  result;

                        SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, sysAppLaunchCmdZLPluginLoad, dataP->ZLPluginFuncPtrs, &result);
                        if (result == errNone)
                        {
                            dataP->cmdBlockTypeEx.cmdBlockType.pZLFuncPtrs = globeP->ZLFuncPtr;
                            dataP->ZLPluginFuncPtrs[ZLPluginFuncPtrRefresh]((ZLPluginLaunchCmdBlockType *)&(dataP->cmdBlockTypeEx));
                        }
                    }
                }
            }
        }
    }
    else
    {
        //͸
        WinSetForeColorRGB(&settingP->formFillColor, &oldForeColor);
        WinSetPatternType(settingP->maskPattern);
        WinPaintRectangle(&globeP->launchViewRectangle, 0);
        WinSetPatternType(blackPattern);
        WinSetForeColorRGB(&settingP->formFrameColor, NULL);

        //ȡʱ
        TimeToAscii(now.hour, now.minute, tfColon24h, globeP->timeStringP);
        stringLength = StrLen(globeP->timeStringP);

        oldFont = FntSetFont(boldFont);

        if (settingP->shouldAddMask)    //ֲ
        {
            WinPaintRectangle(&globeP->datetimeBarRectangle, 0);

            WinSetTextColorRGB(&settingP->textOnMaskColor, &oldTextColor);
            WinPaintChars(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15);
        }
        else
        {
            if (settingP->shouldAddLine)
            {
                WinPaintLine(0, 13, 159, 13);
                WinPaintLine(0, 27, 159, 27);
            }

            WinSetTextColorRGB(&settingP->textColor, &oldTextColor);
            paintCharsWithOutline(globeP->timeStringP, stringLength, 158 - FntCharsWidth(globeP->timeStringP, stringLength), 15, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }
    }

    FntSetFont(oldFont);
    WinSetForeColorRGB(&oldForeColor, NULL);
    WinSetTextColorRGB(&oldTextColor, NULL);

    globeP->justNow = now;
}

//
static void drawPhoneUILaunchBar(SettingType *settingP, GlobeType *globeP)
{
    RectangleType       highlightRectangle;
    ProgramItemInfoType programInfo;
    UInt16              nameLength;
    Coord               x;
    Coord               y;
    Coord               width;
    Coord               height;
    RGBColorType        oldForeColor;
    RGBColorType        oldBackColor;
    RGBColorType        oldTextColor;
    UInt16              i;
    UInt16              row                 = 0;
    UInt16              col                 = 0;
    UInt16              currentProgram      = globeP->beginProgramInPage + globeP->currentCol + 4 * globeP->currentRow;

    //ԭ
    WinSetForeColorRGB(&settingP->formFrameColor, &oldForeColor);
    WinSetBackColorRGB(&settingP->formFillColor, &oldBackColor);
    WinSetTextColorRGB(&settingP->textColor, &oldTextColor);

    if (globeP->isInPhoneView)  //绰ģʽĻײ
    {
        //ĴСλ
        if (settingP->shouldShowTopPrograms)
        {
            globeP->positionRectangle[0].topLeft.y = 121; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
            globeP->positionRectangle[1].topLeft.y = 121; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
            globeP->positionRectangle[2].topLeft.y = 121; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
            globeP->positionRectangle[3].topLeft.y = 121; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;
        }
        else
        {
            globeP->positionRectangle[0].topLeft.y = 131; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 26;
            globeP->positionRectangle[1].topLeft.y = 131; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 26;
            globeP->positionRectangle[2].topLeft.y = 131; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 26;
            globeP->positionRectangle[3].topLeft.y = 131; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 26;
        }
    }
    else    //ģʽ
    {
        if (settingP->shouldAddLine)
        {
            WinPaintLine(0, 146, 159, 146);
        }

        if (settingP->shouldShowSearch && globeP->searchKeywordNum != 0)
        {
            if (globeP->bmpSearchP != NULL)
                WinPaintBitmap(globeP->bmpSearchP, 108, 150);
            StrCopy(globeP->cacheP, globeP->searchKeyword);
            FntSetFont(boldFont);
            paintCharsWithOutline(globeP->cacheP, StrLen(globeP->cacheP), 120, 148, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }

        //״̬ͷҳͷ
        FntSetFont(symbol7Font);
        x =  80 - FntCharWidth('\1') / 2;
        if (globeP->beginProgramInPage == 0)
        {
            if (globeP->currentRow == 0)
            {
                WinSetTextColorRGB(&settingP->formFrameColor, NULL);
                paintCharWithOutline('\1', x, 146, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
                WinSetTextColorRGB(&settingP->textColor, NULL);
            }
            else
            {
                paintCharWithOutline('\3', x, 146, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
            }
        }
        else
        {
            paintCharWithOutline('\1', x, 146, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }
        if (globeP->endProgramInPage == globeP->programListCount - 1 || globeP->programListCount == 0)
        {
            paintCharWithOutline('\4', x, 153, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }
        else
        {
            paintCharWithOutline('\2', x, 153, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
        }
        FntSetFont(stdFont);

        //ĴСλ
        globeP->positionRectangle[0].topLeft.y = 31; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
        globeP->positionRectangle[1].topLeft.y = 31; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
        globeP->positionRectangle[2].topLeft.y = 31; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
        globeP->positionRectangle[3].topLeft.y = 31; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;
    }
    globeP->positionRectangle[0].topLeft.x = 3;
    globeP->positionRectangle[1].topLeft.x = 42;
    globeP->positionRectangle[2].topLeft.x = 82;
    globeP->positionRectangle[3].topLeft.x = 121;

    //ͼ
    WinSetForeColorRGB(&settingP->textColor, NULL);
    WinSetTextColorRGB(&settingP->textColor, NULL);
    if (globeP->programListCount > 0)
    {
        //
        if (globeP->focusedObjectId == gdtIcons && settingP->focusRingStyle > FOCUS_RING_STYLE_ANGLE)
        {
            highlightRectangle = globeP->positionRectangle[globeP->currentCol];
            highlightRectangle.topLeft.y += 38 * globeP->currentRow;
            drawIconFocusedRing(&highlightRectangle, settingP->focusRingStyle, settingP, globeP);
        }

        for (i = globeP->beginProgramInPage; i <= globeP->endProgramInPage; i ++)
        {
            //ȡλ
            x = globeP->positionRectangle[col].topLeft.x;
            y = globeP->positionRectangle[col].topLeft.y + 38 * row;

            //ȡƺͼ
            getProgramItemInfo(settingP->deviceModel, &globeP->programListP[i], &programInfo, globeP);

            //
            if (!globeP->isInPhoneView || settingP->shouldShowTopPrograms)
            {
                nameLength = StrLen(programInfo.nameP);
                width = FntCharsWidth(programInfo.nameP, nameLength);
                if (width <= globeP->positionRectangle[0].extent.x)
                {
                    paintCharsWithOutline(programInfo.nameP, nameLength, x + (globeP->positionRectangle[0].extent.x - width) / 2, y + 24, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
                }
                else
                {
                    winPaintTruncChars(programInfo.nameP, nameLength, x, y + 24, globeP->positionRectangle[0].extent.x, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
                }
            }
            
            //
            if (i == currentProgram && (! globeP->isInPhoneView))
            {
                FntSetFont(boldFont);
                if (settingP->shouldAddMask)
                {
                    WinSetTextColorRGB(&settingP->textOnMaskColor, NULL);
                    if (width <= 100)
                    {
                        WinPaintChars(programInfo.nameP, nameLength, 2, 15);
                    }
                    else
                    {
                        winPaintTruncChars(programInfo.nameP, nameLength, 2, 15, 100, false, NULL, globeP);
                    }
                    WinSetTextColorRGB(&settingP->textColor, NULL);
                }
                else
                {
                    if (width <= 100)
                    {
                        paintCharsWithOutline(programInfo.nameP, nameLength, 2, 15, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
                    }
                    else
                    {
                        winPaintTruncChars(programInfo.nameP, nameLength, 2, 15, 100, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
                    }
                }
                FntSetFont(stdFont);
            }

            //ͼ
            if (programInfo.bmpIconP != NULL)
            {
                BmpGetDimensions(programInfo.bmpIconP, &width, &height, NULL);
                if (width > 32)
                {
                    width /= 2;
                }
                if (height > 22)
                {
                    height = 22;
                }
                WinPaintBitmap(programInfo.bmpIconP, x + 18 - width / 2, y + 13 - height / 2);
            }

            //̱
            if (globeP->programListP[i].VFSNameP[0] != '\0')
            {
                paintCharWithOutline('\32', x + 36 - FntCharWidth('\32'), y, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
            }

            //ݼ
            if (settingP->shouldShowShortCut && globeP->programListP[i].shortcut != chrNull)
            {
                paintCharWithOutline(globeP->programListP[i].shortcut - 32, x + 36 - FntCharWidth(globeP->programListP[i].shortcut), y, settingP->shouldDrawOutline, &settingP->textOutlineColor, globeP);
            }

            //ͼ껺
            if (programInfo.bmpIconP != NULL)
            {
                MemPtrFree(programInfo.bmpIconP);
                programInfo.bmpIconP = NULL;
            }

            col ++;
            if (col > 3)
            {
                row ++;
                col = 0;
            }
        }

        //
        if (globeP->focusedObjectId == gdtIcons && settingP->focusRingStyle < FOCUS_RING_STYLE_ROUND_FRAME)
        {
            highlightRectangle = globeP->positionRectangle[globeP->currentCol];
            highlightRectangle.topLeft.y += 38 * globeP->currentRow;
            drawIconFocusedRing(&highlightRectangle, settingP->focusRingStyle, settingP, globeP);
        }
    }

    WinSetForeColorRGB(&oldForeColor, NULL);
    WinSetBackColorRGB(&oldBackColor, NULL);
    WinSetTextColorRGB(&oldTextColor, NULL);
}

//绰
static void drawPhoneUI(UInt16 code, SettingType *settingP, GlobeType *globeP)
{
    WinPushDrawState();

    WinSetPatternType(blackPattern);

    switch (code)
    {
        case FORM_UPDATE_REFRESH:   //ȫ
            getOperatorName(settingP, globeP);
            getBackgdBitmap(settingP, globeP);
            //
        case frmRedrawUpdateCode:
            //Ƴ
            HsNavRemoveFocusRing(globeP->frmPhoneUIP);

            //ڻлͼ
            WinSetDrawWindow(globeP->offscreenWindowH);

            //״̬
            drawPhoneUIStatusBar(settingP, globeP);

            //Ʊ
            drawPhoneUIBackground(settingP, globeP);

            //
            drawPhoneUIContent(settingP, globeP);

            //
            if
            (
                !globeP->isInPhoneView
                ||
                settingP->shouldShowTopIcons
            )
                drawPhoneUILaunchBar(settingP, globeP);

            //Ļ
            WinSetDrawWindow((WinHandle)globeP->frmPhoneUIP);
            WinCopyRectangle(globeP->offscreenWindowH, (WinHandle)globeP->frmPhoneUIP, &globeP->screenRectangle, 0, 0, winPaint);
            break;
        case FORM_UPDATE_STATUS_BAR_ONLY:
            //ڻлͼ
            WinSetDrawWindow(globeP->offscreenWindowH);
            //״̬
            drawPhoneUIStatusBar(settingP, globeP);
            //Ļ
            WinSetDrawWindow((WinHandle)globeP->frmPhoneUIP);
            WinCopyRectangle(globeP->offscreenWindowH, (WinHandle)globeP->frmPhoneUIP, &globeP->statusBarRectangle, 0, 0, winPaint);
            break;
        case FORM_UPDATE_LAUNCH_BAR_ONLY:
        case FORM_UPDATE_CONTENT:
            //Ƴ
            HsNavRemoveFocusRing(globeP->frmPhoneUIP);

            //ڻлͼ
            WinSetDrawWindow(globeP->offscreenWindowH);

            //Ʊ
            drawPhoneUIBackground(settingP, globeP);

            //
            drawPhoneUIContent(settingP, globeP);

            //
            if
            (
                !globeP->isInPhoneView
                ||
                settingP->shouldShowTopIcons
            )
                drawPhoneUILaunchBar(settingP, globeP);

            //Ļ
            WinSetDrawWindow((WinHandle)globeP->frmPhoneUIP);
            WinCopyRectangle(globeP->offscreenWindowH, (WinHandle)globeP->frmPhoneUIP, &globeP->backgdRectangle, 0, globeP->backgdRectangle.topLeft.y, winPaint);
            break;
    }

    if (globeP->focusedObjectId != gdtIcons)
    {
        HsNavDrawFocusRing(globeP->frmPhoneUIP, globeP->focusedObjectId, hsNavFocusRingNoExtraInfo, getFocusedObjectBounds(globeP->focusedObjectId, globeP), hsNavFocusRingStyleSquare, true);
    }
    
    WinPopDrawState();
}

//ʾʾ
static void drawKeyHoldTip(SettingType *settingP)
{
    RectangleType   r               = {0, 139, 160, 20};
    UInt16          stringLength    = StrLen(STRING_KEY_HOLD_TIP);
    RGBColorType    oldBackColor;
    RGBColorType    oldTextColor;
    FontID          oldFont;

    WinSetBackColorRGB(&settingP->formFrameColor, &oldBackColor);
    WinSetTextColorRGB(&settingP->formFillColor, &oldTextColor);

    WinEraseRectangle(&r, 2);

    oldFont = FntSetFont(boldFont);
    WinPaintChars(STRING_KEY_HOLD_TIP, stringLength, 80 - FntCharsWidth(STRING_KEY_HOLD_TIP, stringLength) / 2, 145);
    FntSetFont(oldFont);

    WinSetBackColorRGB(&oldBackColor, NULL);
    WinSetTextColorRGB(&oldTextColor, NULL);
}

#if 0
#pragma mark == EVENT HANDLER ==
#endif

//лʾģʽ
static void viewSwitch(UInt8 viewMode, SettingType *settingP, GlobeType *globeP)
{
    switch (viewMode)
    {
        case VIEW_MODE_PHONE:   //ת绰
            globeP->isInPhoneView = true;
            getProgramListInCategory(ALL_CATEGORIES, globeP);
            globeP->currentRow = 0;
            globeP->currentCol = 0;
            globeP->beginProgramInPage = 0;
            globeP->endProgramInPage = 3;
            globeP->searchKeywordNum = 0;
            globeP->focusedObjectId = gdtIcons;
            HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
            break;
        case VIEW_MODE_PROGRAM: //ת
            globeP->focusedObjectId = gdtIcons;
            globeP->isInPhoneView = false;
            globeP->searchKeywordNum = 0;
            getProgramListInCategory(settingP->lastCategory, globeP);
            globeP->currentRow = 0;
            globeP->currentCol = 0;
            globeP->beginProgramInPage = 0;
            globeP->endProgramInPage = 11;
            if (globeP->programListCount == 0)
            {
                globeP->endProgramInPage = 0;
                globeP->focusedObjectId = ptriCategory;
            }
            else if (globeP->endProgramInPage > globeP->programListCount - 1)
            {
                globeP->endProgramInPage = globeP->programListCount - 1;
            }
            break;
    }

    globeP->isInSearchMode = false;
}

//ѡť
static Boolean isPenDownOnIcons(Coord x, Coord y, UInt16 *selectedProgram, UInt8 *row, UInt8 *column, SettingType *settingP, GlobeType *globeP)
{
    globeP->positionRectangle[0].topLeft.x = 3;
    globeP->positionRectangle[1].topLeft.x = 42;
    globeP->positionRectangle[2].topLeft.x = 82;
    globeP->positionRectangle[3].topLeft.x = 121;
    if (globeP->isInPhoneView)
    {
        if (settingP->shouldShowTopPrograms)
        {
            globeP->positionRectangle[0].topLeft.y = 121; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
            globeP->positionRectangle[1].topLeft.y = 121; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
            globeP->positionRectangle[2].topLeft.y = 121; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
            globeP->positionRectangle[3].topLeft.y = 121; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;
        }
        else
        {
            globeP->positionRectangle[0].topLeft.y = 131; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 26;
            globeP->positionRectangle[1].topLeft.y = 131; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 26;
            globeP->positionRectangle[2].topLeft.y = 131; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 26;
            globeP->positionRectangle[3].topLeft.y = 131; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 26;
        }
        (*row) = 0;
        for ((*column) = 0; (*column) < 4; (*column) ++)
        {
            if (RctPtInRectangle(x, y, &globeP->positionRectangle[(*column)]))
            {
                (*selectedProgram) = (*column) + globeP->beginProgramInPage;
                return true;
            }
        }
    }
    else
    {
        globeP->positionRectangle[0].topLeft.y = 31; globeP->positionRectangle[0].extent.x = 36; globeP->positionRectangle[0].extent.y = 36;
        globeP->positionRectangle[1].topLeft.y = 31; globeP->positionRectangle[1].extent.x = 36; globeP->positionRectangle[1].extent.y = 36;
        globeP->positionRectangle[2].topLeft.y = 31; globeP->positionRectangle[2].extent.x = 36; globeP->positionRectangle[2].extent.y = 36;
        globeP->positionRectangle[3].topLeft.y = 31; globeP->positionRectangle[3].extent.x = 36; globeP->positionRectangle[3].extent.y = 36;

        for ((*column) = 0; (*column) < 4; (*column) ++)
        {
            for ((*row) = 0; (*row) < 3; (*row) ++)
            {
                if (RctPtInRectangle(x, y, &globeP->positionRectangle[(*column)]))
                {
                    (*selectedProgram) = (*column) + (globeP->beginProgramInPage + ((*row) * 4));
                    if ((*selectedProgram) <= globeP->endProgramInPage)
                    {
                        return true;
                    }
                    else
                    {
                        break;
                    }
                }
                globeP->positionRectangle[(*column)].topLeft.y += 38;
            }
        }
    }
    return false;
}

//Ƿܼ
static Boolean shouldPenDownActivePlugin(EventType *eventP, SettingType *settingP, GlobeType *globeP)
{
    UInt32      i;
    UInt32      result;
    FontID      oldFont = FntSetFont(stdFont);

    if (!settingP->shouldShowCalendar)
    {
        for (i = 0; i < globeP->pluginListCount; i ++)
        {
            if (globeP->pluginListP[i].creatorId == APPLICATION_CREATOR)
            {
                if (RctPtInRectangle(eventP->screenX, eventP->screenY, &globeP->pluginListP[i].pluginRectangle))
                {
                    globeP->pluginParameter.actionType = ACTION_TYPE_TAP;
                    globeP->pluginParameter.screenX = eventP->screenX;
                    globeP->pluginParameter.screenY = eventP->screenY;
                    globeP->pluginParameter.pluginRectangle = globeP->pluginListP[i].pluginRectangle;
                    globeP->pluginParameter.pluginDataP = globeP->pluginListP[i].pluginDataP;
                    SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, LAUNCH_CODE_PLUGIN_ACTION, &globeP->pluginParameter, &result);
                    globeP->pluginListP[i].pluginDataP = globeP->pluginParameter.pluginDataP;
                    return true;
                }
            }
            else if (globeP->pluginListP[i].type == ZL_PLUGIN_LIST_DB_TYPE)
            {
                if (RctPtInRectangle(eventP->screenX, eventP->screenY, &globeP->pluginListP[i].pluginRectangle))
                {
                    ZLPluginData    *dataP = globeP->pluginListP[i].pluginDataP;

                    if (dataP != NULL && dataP->isPluginOpened && dataP->cmdBlockTypeEx.cmdBlockType.dbP != NULL)
                    {
                        UInt32  result;

                        dataP->cmdBlockTypeEx.pluginDataP = (void *)dataP;
                        dataP->eventType.handled = false;
                        dataP->eventType.eventP  = eventP;
                        dataP->cmdBlockTypeEx.cmdBlockType.dataP = (void *)&(dataP->eventType);

                        SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, sysAppLaunchCmdZLPluginLoad, dataP->ZLPluginFuncPtrs, &result);
                        if (result == errNone)
                        {
                            dataP->cmdBlockTypeEx.cmdBlockType.pZLFuncPtrs = globeP->ZLFuncPtr;
                            dataP->ZLPluginFuncPtrs[ZLPluginFuncPtrHandleEvent]((ZLPluginLaunchCmdBlockType *)&(dataP->cmdBlockTypeEx));
                        }

                        return dataP->eventType.handled;
                    }
                }
            }
        }
    }

    FntSetFont(oldFont);

    return false;
}

//Ƿܼ
static Boolean shouldKeyActivePlugin(WChar chr, GlobeType *globeP)
{
    UInt32      i;
    UInt32      result;

    for (i = 0; i < globeP->pluginListCount; i ++)
    {
        if (globeP->pluginListP[i].creatorId == APPLICATION_CREATOR)
        {
            if (globeP->pluginListP[i].activeKey == chr)
            {
                globeP->pluginParameter.actionType = ACTION_TYPE_KEY;
                globeP->pluginParameter.pluginRectangle = globeP->pluginListP[i].pluginRectangle;
                globeP->pluginParameter.pluginDataP = globeP->pluginListP[i].pluginDataP;
                SysAppLaunch(globeP->pluginListP[i].cardNo, globeP->pluginListP[i].dbId, 0, LAUNCH_CODE_PLUGIN_ACTION, &globeP->pluginParameter, &result);
                globeP->pluginListP[i].pluginDataP = globeP->pluginParameter.pluginDataP;

                return true;
            }
        }
    }

    return false;
}

//ɾӿĳ
static void deleteProgramLaunchFromCard(SettingType *settingP)
{
    if (settingP->lastProgramLaunchFromCard.creatorId != 0 && settingP->lastProgramLaunchFromCard.type != 0)
    {
        DmSearchStateType       searchState;

        if (DmGetNextDatabaseByTypeCreator(true, &searchState, settingP->lastProgramLaunchFromCard.type, settingP->lastProgramLaunchFromCard.creatorId, true, &settingP->lastProgramLaunchFromCard.cardNo, &settingP->lastProgramLaunchFromCard.dbId) == errNone)
        {
            DmDeleteDatabase(settingP->lastProgramLaunchFromCard.cardNo, settingP->lastProgramLaunchFromCard.dbId);
            MemSet(&settingP->lastProgramLaunchFromCard, sizeof(ProgramListType), 0);
            PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, sizeof(SettingType), true);
        }
    }
}

//ǷҪʱ
static Boolean shouldUpdateDateTime(GlobeType *globeP)
{
    DateTimeType    now;

    //ȡǰʱ
    TimSecondsToDateTime(TimGetSeconds(), &now);

    if (now.day != globeP->justNow.day || now.minute != globeP->justNow.minute)
    {
        return true;
    }
    return false;
}

//ǷҪ״̬
static Boolean shouldUpdateStatusBar(GlobeType *globeP)
{
    UInt32  currentTicks;

    //ȡǰticks
    currentTicks = TimGetTicks();

    if (currentTicks >= globeP->ticksUpdateStatusBar + 150)
    {
        return true;
    }
    return false;
}

#if 0
#pragma mark -----------------------------------------------------
#endif

//龰ģʽ¼
static void profileEventHandler(SettingType *settingP, GlobeType *globeP)
{
    EventType       event;
    FormType        *frmProfileP;
    ControlType     *pbtnProfilesP[5];
    SoundPreference soundPrefs;
    Int32           i;
    Boolean         isEventHandled;
    Boolean         shouldExit      = false;

    FrmPopupForm(frmProfile);

    do
    {
        EvtGetEvent(&event, evtWaitForever);
        isEventHandled = false;

        if (! SysHandleEvent(&event))
        {
            switch (event.eType)
            {
                case frmLoadEvent:
                    if (event.data.frmLoad.formID == frmProfile)
                    {
                        frmProfileP = (FormType *)FrmInitForm(frmProfile);
                        for (i = 0; i < 5; i ++)
                        {
                            pbtnProfilesP[i] = (ControlType *)GET_OBJECT_POINTER(frmProfileP, pbtnProfile1 + i);
                        }
                        FrmSetActiveForm(frmProfileP);
                        isEventHandled = true;
                    }
                    break;
                case frmOpenEvent:
                    if (event.data.frmOpen.formID == frmProfile)
                    {
                        FrmDrawForm(frmProfileP);
                        for (i = 0; i < 5; i ++)
                        {
                            CtlSetLabel(pbtnProfilesP[i], settingP->profiles[i].nameP);
                        }
                        CtlSetValue(pbtnProfilesP[settingP->lastProfile], 1);
                        HsNavObjectTakeFocus(frmProfileP, pbtnProfile1 + settingP->lastProfile);

                        isEventHandled = true;
                    }
                    break;
                case ctlSelectEvent:
                    settingP->lastProfile = event.data.ctlSelect.controlID - pbtnProfile1;
                    shouldExit = true;
                    break;
            }

            if (! isEventHandled)
            {
                FrmDispatchEvent(&event);
            }
        }
    }while (event.eType != appStopEvent && shouldExit == false);

    FrmReturnToForm(0);

    for (i = soundPrefTypePhone; i < soundPrefTypeLast; i ++)
    {
        TonesLibGetSoundPrefs(globeP->tonesLibRef, i, &soundPrefs);

        switch (i)
        {
            case soundPrefTypePhone:
                soundPrefs.soundOnVolume = settingP->profiles[settingP->lastProfile].phoneSoundPrefs.soundOnVolume;
                soundPrefs.soundOnVibrate = settingP->profiles[settingP->lastProfile].phoneSoundPrefs.soundOnVibrate;
                soundPrefs.soundOffVibrate = settingP->profiles[settingP->lastProfile].phoneSoundPrefs.soundOffVibrate;
                break;
            case soundPrefTypeEmail:
                break;
            case soundPrefTypeCalendar:
                soundPrefs.soundOnVolume = settingP->profiles[settingP->lastProfile].calendarSoundPrefs.soundOnVolume;
                soundPrefs.soundOnVibrate = settingP->profiles[settingP->lastProfile].calendarSoundPrefs.soundOnVibrate;
                soundPrefs.soundOffVibrate = settingP->profiles[settingP->lastProfile].calendarSoundPrefs.soundOffVibrate;
                break;
            case soundPrefTypeIM:
                break;
            case soundPrefTypeMMS:
                break;
            case soundPrefTypeSMS:
                soundPrefs.soundOnVolume = settingP->profiles[settingP->lastProfile].smsSoundPrefs.soundOnVolume;
                soundPrefs.soundOnVibrate = settingP->profiles[settingP->lastProfile].smsSoundPrefs.soundOnVibrate;
                soundPrefs.soundOffVibrate = settingP->profiles[settingP->lastProfile].smsSoundPrefs.soundOffVibrate;
                break;
        }

        TonesLibSetSoundPrefs(globeP->tonesLibRef, i, &soundPrefs);
    }
}

//˵¼
static Boolean categoryListPopupEventHandler(SettingType *settingP, GlobeType *globeP)
{
    Int16       selectedIndex   = LstPopupList(globeP->lstCategoryP);

    if (selectedIndex == noListSelection)
    {
        return false;
    }
    else
    {
        settingP->lastCategory = getCategoryId(selectedIndex, 0, globeP);
    }

    return true;
}

//ؼѡ¼
static Boolean controlSelectEventHandler(UInt16 objectId, SettingType *settingP, GlobeType *globeP)
{
    Boolean         isEventHandled  = false;

    switch (objectId)
    {
        case gdtCarrier:
            if(AttnGetCounts(0 ,0, NULL, NULL) != 0)
            {
                AttnListOpen();
            }
            else
            {
                EvtEnqueueKey(vchrMenu, 0, 8);
            }
            isEventHandled = true;
            break;
        case gdtSound:
            {
                DmSearchStateType       stateInfo;
                UInt16                  cardNo;
                LocalID                 dbID;

                if (DmGetNextDatabaseByTypeCreator(true, &stateInfo, sysFileTPanel, 'HsRN', true, &cardNo, &dbID) == errNone)
                {
                    SysUIAppSwitch(cardNo, dbID, sysAppLaunchCmdPanelCalledFromApp, NULL);
                }
            }
            isEventHandled = true;
            break;
        case gdtBlueTooth:
            {
                DmSearchStateType       stateInfo;
                UInt16                  cardNo;
                LocalID                 dbID;

                if (DmGetNextDatabaseByTypeCreator(true, &stateInfo, 'appl', 'BTMr', true, &cardNo, &dbID) == errNone)
                {
                    SysUIAppSwitch(cardNo, dbID, sysAppLaunchCmdPanelCalledFromApp, NULL);
                }
            }
            isEventHandled = true;
            break;
        case gdtSignal:
            {
                DmSearchStateType       stateInfo;
                UInt16                  cardNo;
                LocalID                 dbID;

                if (DmGetNextDatabaseByTypeCreator(true, &stateInfo, sysFileTPanel, sysFileCNetworkPanel, true, &cardNo, &dbID) == errNone)
                {
                    SysUIAppSwitch(cardNo, dbID, sysAppLaunchCmdPanelCalledFromApp, NULL);
                }
            }
            isEventHandled = true;
            break;
        case gdtDatetime:
            settingP->shouldShowCalendar = ! settingP->shouldShowCalendar;
            FrmUpdateForm(frmPhoneUI, FORM_UPDATE_ALL);
            isEventHandled = true;
            break;
        case ptriCategory:
            CtlHitControl(globeP->ptriCategoryP);
            isEventHandled = true;
            break;
        case gdtIcons:
            if
            (
                (
                    !globeP->isInPhoneView
                    ||
                    settingP->shouldShowTopIcons
                )
                &&
                (globeP->programListCount != 0)
            )
            {
                objectId = globeP->beginProgramInPage + globeP->currentRow * 4 + globeP->currentCol;
                launchProgram(&globeP->programListP[objectId], objectId, settingP, globeP);
                isEventHandled = true;
            }
            break;
    }

    return isEventHandled;
}

//˵¼
static Boolean menuItemEventHandler(UInt16 *formUpdateCodeP, UInt16 menuItemId, SettingType *settingP, GlobeType *globeP)
{
    FormType                *frmAboutP;
    Boolean                 isEventHandled      = false;

    switch (menuItemId)
    {
        case menuitemPhoneON:
            PhnLibSetModulePower(globeP->hsPhoneLibRef, true);
            break;
        case menuitemPhoneOFF:
            PhnLibSetModulePower(globeP->hsPhoneLibRef, false);
            break;
        case menuitemDropNetwork:
            HsNetworkDropConnection();
            break;
        case menuitemCategory:
            programManagement(globeP, MANAGEMENT_CATEGORY, settingP);

            //ˢ·б
            LstSetListChoices(globeP->lstCategoryP, NULL, 0);
            freeGlobeCategoryList(globeP);
            buildCategoryList(globeP);
            LstSetListChoices(globeP->lstCategoryP, globeP->categoryListPP, globeP->categoryCount);
            LstSetSelection(globeP->lstCategoryP, 0);
            LstSetHeight(globeP->lstCategoryP, globeP->categoryCount);
            LstSetPosition(globeP->lstCategoryP, 1, 157 - 11 * globeP->categoryCount);
            CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
            settingP->lastCategory = ALL_CATEGORIES;

            if (! globeP->isInPhoneView)
            {
                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
            }
            (*formUpdateCodeP) = FORM_UPDATE_ALL;
            isEventHandled = true;
            break;
        case menuitemCategoryAndShortcut:
            programManagement(globeP, MANAGEMENT_CATEGORY_AND_SHORT_CUT, settingP);
            if (globeP->isInPhoneView)
            {
                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
            }
            else
            {
                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
            }
            (*formUpdateCodeP) = FORM_UPDATE_ALL;
            isEventHandled = true;
            break;
        case menuitemLaunchDefaultLauncher:
            LaunchWithCommand('appl', sysFileCLauncher, sysAppLaunchCmdNormalLaunch, NULL);
            isEventHandled = true;
            break;
        case menuitemSetting:
            showSettingUI(settingP, globeP);

            (*formUpdateCodeP) = FORM_UPDATE_REFRESH;
            isEventHandled = true;
            break;
        case menuitemPreference:
            LaunchWithCommand('appl', PREF_CREATOR, sysAppLaunchCmdNormalLaunch, NULL);
            isEventHandled = true;
            break;
        case menuitemRefreshProgramList:
            buildProgramList(BUILD_LIST_MODE_REFRESH, true, true, globeP);

            //ˢ½
            if (globeP->isInPhoneView)
            {
                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
            }
            else
            {
                LstSetSelection(globeP->lstCategoryP, 0);
                CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
                settingP->lastCategory = ALL_CATEGORIES;
                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
            }
            break;
        case menuitemRebuildAll:
            if (FrmCustomAlert(alertConfirmation, STRING_REBUILD_LIST_CONFIRM, "", "") == 0)
            {
                buildProgramList(BUILD_LIST_MODE_INIT, true, true, globeP);

                LstSetListChoices(globeP->lstCategoryP, NULL, 0);
                freeGlobeCategoryList(globeP);
                buildCategoryList(globeP);
                LstSetListChoices(globeP->lstCategoryP, globeP->categoryListPP, globeP->categoryCount);
                LstSetSelection(globeP->lstCategoryP, 0);
                LstSetHeight(globeP->lstCategoryP, globeP->categoryCount);
                LstSetPosition(globeP->lstCategoryP, 1, 157 - 11 * globeP->categoryCount);
                CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
                settingP->lastCategory = ALL_CATEGORIES;

                //ˢ½
                if (globeP->isInPhoneView)
                {
                    viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                }
                else
                {
                    LstSetSelection(globeP->lstCategoryP, 0);
                    CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
                    viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                }
            }
            isEventHandled = true;
            break;
        case menuitemResetSystem:
            if (FrmCustomAlert(alertConfirmation, STRING_CONFIRM_RESET, "", "") == 0)
            {
                SysReset();
            }
            break;
        case menuitemAbout:
            frmAboutP = FrmInitForm(frmAbout);
            FrmDoDialog(frmAboutP);
            FrmDeleteForm(frmAboutP);
            (*formUpdateCodeP) = FORM_UPDATE_ALL;
            isEventHandled = true;
            break;
    }

    return isEventHandled;
}

//绰绰¼
static Boolean frmPhoneUICustomEventHandler(EventType *eventP, SettingType *settingP, GlobeType *globeP)
{
    Boolean         isCustomEventHandled    = false;
    UInt16          formUpdateCode          = FORM_UPDATE_NO_UPDATE;

    switch (eventP->eType)
    {
        case PHONE_EVENT_NETWORK_SEARCHING:
            StrCopy(globeP->operatorNameP, STRING_NETWORK_SEARCHING);
            formUpdateCode = FORM_UPDATE_STATUS_BAR_ONLY;
            isCustomEventHandled = true;
            break;
        case PHONE_EVENT_POWER_ON:
            StrCopy(globeP->operatorNameP, STRING_PHONE_ON);
            formUpdateCode = FORM_UPDATE_STATUS_BAR_ONLY;
            isCustomEventHandled = true;
            break;
        case PHONE_EVENT_POWER_OFF:
            StrCopy(globeP->operatorNameP, STRING_PHONE_OFF);
            formUpdateCode = FORM_UPDATE_STATUS_BAR_ONLY;
            isCustomEventHandled = true;
            break;
        case PHONE_EVENT_HOME:
            getOperatorName(settingP, globeP);
            formUpdateCode = FORM_UPDATE_STATUS_BAR_ONLY;
            isCustomEventHandled = true;
            break;
        case PHONE_EVENT_OTHER:
        case NOTIFY_EVENT_DB:
            formUpdateCode = FORM_UPDATE_ALL;
            isCustomEventHandled = true;
            break;
        case NOTIFY_EVENT_CARD_MOUNT:
            globeP->isVolumeMounted = true;
            if (settingP->shouldAutoUpdateProgramList)
            {
                buildProgramList(BUILD_LIST_MODE_REFRESH, false, true, globeP);
            }
            //ˢ½
            if (globeP->isInPhoneView)
            {
                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
            }
            else
            {
                LstSetSelection(globeP->lstCategoryP, 0);
                CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
                settingP->lastCategory = ALL_CATEGORIES;
                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
            }
            formUpdateCode = FORM_UPDATE_ALL;
            isCustomEventHandled = true;
            break;
        case NOTIFY_EVENT_CARD_UNMOUNT:
            globeP->isVolumeMounted = false;
            //ˢ½
            if (globeP->isInPhoneView)
            {
                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
            }
            else
            {
                LstSetSelection(globeP->lstCategoryP, 0);
                CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
                settingP->lastCategory = ALL_CATEGORIES;
                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
            }
            formUpdateCode = FORM_UPDATE_ALL;
            isCustomEventHandled = true;
            break;
        case NOTIFY_EVENT_WAKE_UP:
            drawPhoneUI(FORM_UPDATE_CONTENT, settingP, globeP);
            break;
        case NOTIFY_EVENT_ALERT:
            formUpdateCode = FORM_UPDATE_STATUS_BAR_ONLY;
            isCustomEventHandled = true;
    }

    if (isCustomEventHandled)
    {
        FrmUpdateForm(frmPhoneUI, formUpdateCode);
    }

    return isCustomEventHandled;
}

//绰¼
static Boolean frmPhoneUIAppEventHandler(EventType *eventP, SettingType *settingP, GlobeType *globeP)
{
    UInt8           row, column;
    Int32           i;
    UInt16          selectedProgram;
    Boolean         isEventHandled      = false;
    UInt16          formUpdateCode      = FORM_UPDATE_NO_UPDATE;

    switch (eventP->eType)
    {
        case frmLoadEvent:
            if (eventP->data.frmLoad.formID == frmPhoneUI)
            {
                //봰
                globeP->frmPhoneUIP = FrmInitForm(frmPhoneUI);
                globeP->lstCategoryP = (ListType *)GET_OBJECT_POINTER(globeP->frmPhoneUIP, lstCategory);
                globeP->ptriCategoryP = (ControlType *)GET_OBJECT_POINTER(globeP->frmPhoneUIP, ptriCategory);

                isEventHandled = true;
            }
            break;
        case frmOpenEvent:
            if (eventP->data.frmOpen.formID == frmPhoneUI)
            {
                //ȡͼ
                if (settingP->shouldShowBackgd)
                {
                    getBackgdBitmap(settingP, globeP);
                }

                //Gadget
//              PmSysGadgetStatusGadgetTypeSet(globeP->pmGadgetLibRef, (void *)globeP->frmPhoneUIP, gdtBlueTooth, pmSysGadgetStatusGadgetBt);
//              PmSysGadgetStatusGadgetTypeSet(globeP->pmGadgetLibRef, (void *)globeP->frmPhoneUIP, gdtSignal, pmSysGadgetStatusGadgetSignal);
//              PmSysGadgetStatusGadgetTypeSet(globeP->pmGadgetLibRef, (void *)globeP->frmPhoneUIP, gdtBattery, pmSysGadgetStatusGadgetBattery);

                //ȡʶ
                getOperatorName(settingP, globeP);

                //
                FrmSetActiveForm(globeP->frmPhoneUIP);

                //
                FrmDrawForm(globeP->frmPhoneUIP);

                //ûͼģʽ
                WinSetDrawMode(winOverlay);

                //ʼ
                globeP->focusedObjectId = gdtIcons;
                globeP->isInPhoneView = true;
                getProgramListInCategory(ALL_CATEGORIES, globeP);
                globeP->beginProgramInPage = 0;
                globeP->endProgramInPage = 3;
                globeP->currentRow = 0;
                globeP->currentCol = 0;
                buildCategoryList(globeP);
                LstSetListChoices(globeP->lstCategoryP, globeP->categoryListPP, globeP->categoryCount);

                column = 0;
                for (row = 0; row < globeP->categoryCount; row ++)
                {
                    if (settingP->lastCategory == globeP->categoryListMappingP[row])    //ҵ
                    {
                        column = 1;
                        LstSetSelection(globeP->lstCategoryP, row);
                        CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, row));
                    }
                }
                if (column == 0)    //ûƥ
                {
                    settingP->lastCategory = ALL_CATEGORIES;
                    LstSetSelection(globeP->lstCategoryP, 0);
                    CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, 0));
                }

                LstSetHeight(globeP->lstCategoryP, globeP->categoryCount);
                LstSetPosition(globeP->lstCategoryP, 1, 157 - 11 * globeP->categoryCount);

                //ȡб
                getPluginList(&globeP->pluginListP, &globeP->pluginListCount, true, globeP);

                //drawPhoneUI(frmRedrawUpdateCode, settingP);

                WinSetBackColorRGB(&settingP->formFillColor, NULL);

                globeP->isFormLoaded = true;
                isEventHandled = true;
            }
            break;
        case frmUpdateEvent:
            if (eventP->data.frmUpdate.formID == frmPhoneUI)
            {
                drawPhoneUI(eventP->data.frmUpdate.updateCode, settingP, globeP);

                isEventHandled = true;
            }
            break;
        case frmObjectFocusTakeEvent:
            if (eventP->data.frmObjectFocusTake.formID == frmPhoneUI)
            {
                HsNavRemoveFocusRing(globeP->frmPhoneUIP);
                switch (eventP->data.frmObjectFocusTake.objectID)
                {
                    case gdtIcons:
                        break;
                    default:
                        HsNavDrawFocusRing
                        (
                            globeP->frmPhoneUIP,
                            globeP->focusedObjectId,
                            hsNavFocusRingNoExtraInfo,
                            getFocusedObjectBounds(eventP->data.frmObjectFocusTake.objectID, globeP),
                            hsNavFocusRingStyleSquare,
                            true
                        );
                        break;
                }
                isEventHandled = true;
            }
            break;
        case menuOpenEvent:
            showHideMenuitem(globeP);
            globeP->isKeyDownDectected = false;
            break;
        case menuEvent:
            isEventHandled = menuItemEventHandler(&formUpdateCode, eventP->data.menu.itemID, settingP, globeP);
            /*if (formUpdateCode != FORM_UPDATE_NO_UPDATE)
            {
                HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
            }*/
            HsNavObjectTakeFocus(globeP->frmPhoneUIP, globeP->focusedObjectId);
            break;
//      case frmGadgetEnterEvent:
//          if (eventP->data.gadgetEnter.gadgetID == gdtBattery)
//          {
//              globeP->isGadgetActived = true;
//          }
//          break;
        case winEnterEvent:
            if (eventP->data.winEnter.enterWindow == (WinHandle)globeP->frmPhoneUIP /* || globeP->isGadgetActived */)
            {
                formUpdateCode = FORM_UPDATE_ALL;
                globeP->isMenuActived = false;
            }
//          globeP->isGadgetActived = false;
            break;
        case popSelectEvent:
            if (eventP->data.popSelect.selection != noListSelection)
            {
                if (settingP->shouldSkipEmptyCategory && globeP->isInSearchMode == false && globeP->isPopSelectEventGeneratedByKey)
                {
                    globeP->isPopSelectEventGeneratedByKey = false;
                    i = eventP->data.popSelect.selection;

                    do
                    {
                        LstSetSelection(globeP->lstCategoryP, i);
                        eventP->data.popSelect.selection = i;
                        settingP->lastCategory = getCategoryId(i, globeP->categoryCount, globeP);
                        getProgramListInCategory(settingP->lastCategory, globeP);

                        i ++;
                        if (i >= globeP->categoryCount)
                        {
                            if (settingP->shouldSkipAllCategory)
                            {
                                i = 1;
                            }
                            else
                            {
                                i = 0;
                            }
                        }
                    }while (globeP->programListCount == 0);

                    CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, LstGetSelection(globeP->lstCategoryP)));
                }
                else
                {
                    CtlSetLabel(globeP->ptriCategoryP, LstGetSelectionText(globeP->lstCategoryP, eventP->data.popSelect.selection));
                    settingP->lastCategory = getCategoryId(eventP->data.popSelect.selection, globeP->categoryCount, globeP);

                    if (globeP->isInSearchMode)
                    {
                        if (globeP->searchKeywordNum != 0)
                            searchProgramInCategory(globeP->searchKeyword, settingP->lastCategory, globeP);
                    }
                    else
                    {
                        getProgramListInCategory(settingP->lastCategory, globeP);
                    }
                }
                globeP->currentRow = 0;
                globeP->currentCol = 0;
                globeP->beginProgramInPage = 0;
                globeP->endProgramInPage = 11;
                if (globeP->programListCount == 0)
                {
                    globeP->endProgramInPage = 0;
                }
                else if (globeP->endProgramInPage > globeP->programListCount - 1)
                {
                    globeP->endProgramInPage = globeP->programListCount - 1;
                }
                formUpdateCode = FORM_UPDATE_ALL;

                if (globeP->programListCount != 0)
                {
                    globeP->focusedObjectId = gdtIcons;
                    HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
                }
            }
            break;
        case fldEnterEvent:
            break;
        case fldChangedEvent:
            break;
        case penDownEvent:
            globeP->isPenDownDectected = true;
            if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtCarrier, globeP)))
            {
                globeP->penDownObjectId = gdtCarrier;
                isEventHandled = true;
            }
            if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtSound, globeP)))
            {
                globeP->penDownObjectId = gdtSound;
                isEventHandled = true;
            }
            else if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtBlueTooth, globeP)))
            {
                globeP->penDownObjectId = gdtBlueTooth;
                isEventHandled = true;
            }
            else if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtSignal, globeP)) && (PhnLibModulePowered(globeP->hsPhoneLibRef) == phnPowerOn))
            {
                globeP->penDownObjectId = gdtSignal;
                isEventHandled = true;
            }
            else if (globeP->isInPhoneView && settingP->shouldShowDateTime && RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtDatetime, globeP)))
            {
                globeP->penDownObjectId = gdtDatetime;
                isEventHandled = true;
            }
            else if
            (
                (
                    !globeP->isInPhoneView
                    ||
                    settingP->shouldShowTopIcons
                )
                &&
                isPenDownOnIcons(eventP->screenX, eventP->screenY, &selectedProgram, &row, &column, settingP, globeP))    //ƶ
            {
                globeP->currentRow = row;
                globeP->currentCol = column;
                globeP->focusedObjectId = gdtIcons;
                globeP->penDownObjectId = gdtIcons;
                formUpdateCode = FORM_UPDATE_CONTENT;
                isEventHandled = true;
            }

            if (! isEventHandled && globeP->isInPhoneView)
            {
                formUpdateCode = FORM_UPDATE_CONTENT;
                isEventHandled = shouldPenDownActivePlugin(eventP, settingP, globeP);
            }

            break;
        case penUpEvent:
            if (globeP->isPenDownDectected)
            {
                globeP->isPenDownDectected = false;

                if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtCarrier, globeP)) &&
                    globeP->penDownObjectId == gdtCarrier)      //˵
                {
                    isEventHandled = controlSelectEventHandler(gdtCarrier, settingP, globeP);
                }
                if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtSound, globeP)) &&
                    globeP->penDownObjectId == gdtSound)        //ϵͳ
                {
                    isEventHandled = controlSelectEventHandler(gdtSound, settingP, globeP);
                }
                if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtBlueTooth, globeP)) &&
                    globeP->penDownObjectId == gdtBlueTooth)        //ϵͳ
                {
                    isEventHandled = controlSelectEventHandler(gdtBlueTooth, settingP, globeP);
                }
                if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtSignal, globeP)) &&
                    globeP->penDownObjectId == gdtSignal)       //ϵͳ
                {
                    isEventHandled = controlSelectEventHandler(gdtSignal, settingP, globeP);
                }
                else if (RctPtInRectangle(eventP->screenX, eventP->screenY, getFocusedObjectBounds(gdtDatetime, globeP)) && globeP->isInPhoneView &&
                        globeP->penDownObjectId == gdtDatetime) //
                {
                    if (settingP->shouldShowDateTime)
                    {
                        isEventHandled = controlSelectEventHandler(gdtDatetime, settingP, globeP);
                    }
                }
                else if (isPenDownOnIcons(eventP->screenX, eventP->screenY, &selectedProgram, &row, &column, settingP, globeP) &&
                        globeP->penDownObjectId == gdtIcons)    //ѡ˳
                {
                    if (row == globeP->currentRow && column == globeP->currentCol)
                    {
                        launchProgram(&globeP->programListP[selectedProgram], selectedProgram, settingP, globeP);
                        isEventHandled = true;
                    }
                }
                else if (globeP->focusedObjectId == gdtIcons)       //ƶл
                {
                    if (eventP->screenX >= 77 && eventP->screenX <= 82)
                    {
                        if (eventP->screenY >= 146 && eventP->screenY <= 152 && (! globeP->isInPhoneView))  //
                        {
                            if (globeP->programListCount > 0)
                            {
                                if (globeP->currentRow > 0) //Уһ
                                {
                                    globeP->currentRow --;
                                    formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
                                }
                                else if (globeP->beginProgramInPage > 0)    //Уδͷбһ
                                {
                                    globeP->beginProgramInPage -= 4;
                                    globeP->endProgramInPage = globeP->beginProgramInPage + 11;
                                    if (globeP->endProgramInPage > globeP->programListCount - 1)    //ûҪж
                                    {
                                        globeP->endProgramInPage = globeP->programListCount - 1;
                                    }
                                    formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
                                }
                                else    //УѾͷָ绰ģʽ
                                {
                                    viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                                    formUpdateCode = FORM_UPDATE_ALL;
                                }
                            }
                            else    //ݣָ绰ģʽ
                            {
                                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                                formUpdateCode = FORM_UPDATE_ALL;
                            }
                            isEventHandled = true;
                        }
                        else if (eventP->screenY >= 153)    //
                        {
                            if (globeP->isInPhoneView)  //绰棬л
                            {
                                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                                formUpdateCode = FORM_UPDATE_ALL;
                                isEventHandled = true;
                            }
                            else if (globeP->programListCount > 0)  //
                            {
                                if (globeP->currentRow < (globeP->endProgramInPage - globeP->beginProgramInPage) / 4)   //ĩУһ
                                {
                                    globeP->currentRow ++;
                                    formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
                                }
                                else if (globeP->endProgramInPage < globeP->programListCount - 1)   //ĩУδͷбһ
                                {
                                    globeP->beginProgramInPage += 4;
                                    globeP->endProgramInPage += 4;
                                    if (globeP->endProgramInPage > globeP->programListCount - 1)
                                    {
                                        globeP->endProgramInPage = globeP->programListCount - 1;
                                    }
                                    formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
                                }
                                if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol > globeP->programListCount - 1)
                                {
                                    globeP->currentCol = (globeP->endProgramInPage - globeP->beginProgramInPage) % 4;
                                }
                                isEventHandled = true;
                            }
                        }
                    }
                }

                if (! isEventHandled && globeP->isInPhoneView)
                {
                    isEventHandled = shouldPenDownActivePlugin(eventP, settingP, globeP);
                    formUpdateCode = FORM_UPDATE_CONTENT;
                }

                globeP->penDownObjectId = 0;
            }
            break;
    }

    if (formUpdateCode != FORM_UPDATE_NO_UPDATE)
    {
        FrmUpdateForm(frmPhoneUI, formUpdateCode);
    }
    return isEventHandled;
}

//´
static Boolean arrowKeyEventHandler(WChar chrKey, UInt16 *formUpdateCodeP, SettingType *settingP, GlobeType *globeP)
{
    switch(chrKey)
    {
        case chrUpArrow:
        if (globeP->isInPhoneView)  //绰ģʽ
        {
            if (globeP->focusedObjectId == gdtIcons && settingP->shouldShowDateTime)
            {
                globeP->focusedObjectId = gdtDatetime;
                *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                HsNavObjectTakeFocus(globeP->frmPhoneUIP, globeP->focusedObjectId);
            }
        }
        else    //ǵ绰ģʽ
        {
            if (globeP->focusedObjectId == gdtIcons)
            {
                if (globeP->programListCount > 0)
                {
                    if (globeP->currentRow > 0) //Уһ
                    {
                        globeP->currentRow --;
                        *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    }
                    else if (globeP->beginProgramInPage > 0)    //Уδͷб
                    {
                        if (globeP->beginProgramInPage < 12)
                        {
                            globeP->currentRow = (globeP->beginProgramInPage - 1) / 4;
                            globeP->beginProgramInPage = 0;
                        }
                        else
                        {
                            globeP->beginProgramInPage -= 12;
                            globeP->currentRow = 2;
                        }
                        globeP->endProgramInPage = globeP->beginProgramInPage + 11;
                        if (globeP->endProgramInPage > globeP->programListCount - 1)
                        {
                            globeP->endProgramInPage = globeP->programListCount - 1;
                        }
                        *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    }
                    else    //УѾͷָ绰ģʽ
                    {
                        viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                        *formUpdateCodeP = FORM_UPDATE_ALL;
                    }
                }
                else    //ݣָ绰ģʽ
                {
                    viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                    *formUpdateCodeP = FORM_UPDATE_ALL;
                }
            }
            else
            {
                if (globeP->programListCount > 0)
                {
                    globeP->focusedObjectId = gdtIcons;
                    HsNavObjectTakeFocus(globeP->frmPhoneUIP, gdtIcons);
                    *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                }
                else    //ָ绰ģʽ
                {
                    viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                    *formUpdateCodeP = FORM_UPDATE_ALL;
                }
            }
        }
        return true;

        case chrDownArrow:
        if (globeP->isInPhoneView)  //绰
        {
            switch (globeP->focusedObjectId)
            {
                case gdtDatetime:
                    globeP->focusedObjectId = gdtIcons;
                    *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    break;
                case gdtIcons:
                    viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                    *formUpdateCodeP = FORM_UPDATE_ALL;
                    break;
                default:
                    globeP->focusedObjectId = gdtDatetime;
                    break;
            }
            HsNavObjectTakeFocus(globeP->frmPhoneUIP, globeP->focusedObjectId);
        }
        else
        {
            if (globeP->programListCount > 0 && globeP->focusedObjectId == gdtIcons)
            {
                if (! globeP->isInPhoneView)    //ǵ绰ģʽ
                {
                    if (globeP->currentRow < (globeP->endProgramInPage - globeP->beginProgramInPage) / 4)   //ĩУһ
                    {
                        globeP->currentRow++;
                        *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    }
                    else if (globeP->endProgramInPage < globeP->programListCount - 1)   //ĩУδͷбһ
                    {
                        globeP->beginProgramInPage += 12;
                        globeP->endProgramInPage += 12;
                        if (globeP->endProgramInPage > globeP->programListCount - 1)
                        {
                            globeP->endProgramInPage = globeP->programListCount - 1;
                        }
                        globeP->currentRow = 0;
                        *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    }
                    if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol > globeP->programListCount - 1)
                    {
                        globeP->currentCol = (globeP->endProgramInPage - globeP->beginProgramInPage) % 4;
                    }
                }
                else    //绰ģʽתģʽ
                {
                    viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                    *formUpdateCodeP = FORM_UPDATE_ALL;
                }
            }
        }
        return true;

        case chrLeftArrow:
        if (globeP->focusedObjectId == gdtIcons)
        {
            if
            (
                !globeP->isInPhoneView
                ||
                settingP->shouldShowTopIcons
            )
            {
                if (globeP->programListCount > 0)
                {
                    if (globeP->currentCol > 0) //
                    {
                        globeP->currentCol --;
                        *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    }
                    else if (globeP->isInPhoneView) //ﲻжshouldShowtopIcons̶ʾ8
                    {
                        if (globeP->endProgramInPage < globeP->programListCount - 1)
                        {
                            globeP->currentCol = 3;
                            if (globeP->beginProgramInPage == 0)
                            {
                                globeP->beginProgramInPage = 4;
                                globeP->endProgramInPage = 8;
                            }
                            else
                            {
                                globeP->beginProgramInPage = 0;
                                globeP->endProgramInPage = 4;
                            }
                            if (globeP->endProgramInPage > globeP->programListCount - 1)
                            {
                                globeP->endProgramInPage = globeP->programListCount - 1;
                            }
                            *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                        }
                    }
                    else    //ףҲڵ绰ģʽ
                    {
                        if (globeP->currentRow != 0)    //УҪƶб
                        {
                            globeP->currentCol = 3;
                            globeP->currentRow --;
                        }
                        else if (globeP->beginProgramInPage != 0) //УҪƶб
                        {
                            if (globeP->beginProgramInPage < 12)
                            {
                                globeP->currentRow = (globeP->beginProgramInPage - 1) / 4;
                                globeP->beginProgramInPage = 0;
                            }
                            else
                            {
                                globeP->beginProgramInPage -= 12;
                                globeP->currentRow = 2;
                            }
                            globeP->endProgramInPage = globeP->beginProgramInPage + 11;
                            if (globeP->endProgramInPage > globeP->programListCount - 1)
                            {
                                globeP->endProgramInPage = globeP->programListCount - 1;
                            }
                            globeP->currentCol = 3;
                        }
                        *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                    }
                }
                return true;
            }
        }
        return false;

        case chrRightArrow:
        if (globeP->focusedObjectId == gdtIcons)
        {
            if
            (
                !globeP->isInPhoneView
                ||
                settingP->shouldShowTopIcons
            )
            {
                if (globeP->programListCount > 0)
                {
                    if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol < globeP->programListCount - 1)    //
                    {
                        if (globeP->currentCol < 3) //δĩ1λ
                        {
                            globeP->currentCol++;
                            *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                        }
                        else if (globeP->isInPhoneView) //ﲻжshouldShowtopIcons̶ʾ8
                        {
                            if (globeP->endProgramInPage < globeP->programListCount - 1)
                            {
                                globeP->currentCol = 0;
                                if (globeP->beginProgramInPage == 0)
                                {
                                    globeP->beginProgramInPage = 4;
                                    globeP->endProgramInPage = 8;
                                }
                                else
                                {
                                    globeP->beginProgramInPage = 0;
                                    globeP->endProgramInPage = 4;
                                }
                                if (globeP->endProgramInPage > globeP->programListCount - 1)
                                {
                                    globeP->endProgramInPage = globeP->programListCount - 1;
                                }
                                *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                            }
                        }
                        else    //ĩҲڵ绰ģʽ
                        {
                            if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol < globeP->endProgramInPage)    //Ҫƶб
                            {
                                //һ
                                globeP->currentCol = 0;
                                globeP->currentRow ++;
                                *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                            }
                            else if (globeP->endProgramInPage < globeP->programListCount - 1)   //ƶб
                            {
                                //ҳƶ3
                                globeP->beginProgramInPage += 12;
                                globeP->endProgramInPage += 12;
                                if (globeP->endProgramInPage > globeP->programListCount - 1)
                                {
                                    globeP->endProgramInPage = globeP->programListCount - 1;
                                }
                                globeP->currentRow = 0;
                                globeP->currentCol = 0;
                                *formUpdateCodeP = FORM_UPDATE_LAUNCH_BAR_ONLY;
                            }
                        }
                    }
                }
                return true;
            }
        }
        return false;

        default:
        return false;
    }
}

static void keyPressEventHandler(WChar chr, UInt16 keyCode, UInt16 *formUpdateCodeP, SettingType *settingP, GlobeType *globeP)
{
    if (((keyCode >= keyA && keyCode <= keyZ) || chr == keyZero || keyCode == keySpace))   //
    {
        //һΰڵ绰棬ת
        if (globeP->searchKeywordNum == 0)
        {
            globeP->searchKeywordTime = TimGetSeconds();

            if (globeP->isInPhoneView)
            {
                globeP->isSearchBeginAtPhoneView = true;
                globeP->isInPhoneView = false;
                viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                *formUpdateCodeP = FORM_UPDATE_ALL;
            }
            else
            {
                globeP->isSearchBeginAtPhoneView = false;
            }
        }
        else    //ж
        {
            UInt32  oldTime = globeP->searchKeywordTime;

            globeP->searchKeywordTime = TimGetSeconds();

            if ((globeP->searchKeywordTime - oldTime) > 2)
                globeP->searchKeywordNum = 0;
        }

        if (globeP->searchKeywordNum < 5)
        {

            if (chr < 128)
            {
                globeP->searchKeyword[globeP->searchKeywordNum++] = (Char)chr;
                globeP->searchKeyword[globeP->searchKeywordNum] = '\0';
            }
            else
            {
                if (globeP->searchKeywordNum < 4)
                {
                    MemMove(&globeP->searchKeyword[globeP->searchKeywordNum], &chr, 2);
                    globeP->searchKeywordNum += 2;
                    globeP->searchKeyword[globeP->searchKeywordNum] = '\0';
                }
            }

            if (settingP->shouldSearchInGlobe)
            {
                searchProgramInCategory(globeP->searchKeyword, ALL_CATEGORIES, globeP);
            }
            else
            {
                searchProgramInCategory(globeP->searchKeyword, settingP->lastCategory, globeP);
            }

            globeP->currentRow = 0;
            globeP->currentCol = 0;
            globeP->beginProgramInPage = 0;
            globeP->endProgramInPage = 11;

            if (globeP->programListCount == 0)
            {
                globeP->endProgramInPage = 0;
            }
            else if (globeP->endProgramInPage > globeP->programListCount - 1)
            {
                globeP->endProgramInPage = globeP->programListCount - 1;
            }

            if (globeP->programListCount == 1 && settingP->shouldLaunchTheOnlyMatchedProgram)   //Ψһĳ
            {
                launchProgram(&globeP->programListP[0], 0, settingP, globeP);
            }

            globeP->isInSearchMode = true;
            *formUpdateCodeP = FORM_UPDATE_ALL;
        }
    }
}

//¼
static Boolean frmPhoneUIKeyEventHandler(EventType *eventP, SettingType *settingP, GlobeType *globeP)
{
    UInt16          selectedProgram;
    Boolean         isShortCutExist;
    Boolean         isKeyHandled        = false;

    if (! globeP->isMenuActived)
    {
        WChar           chr;
        UInt16          formUpdateCode  = FORM_UPDATE_NO_UPDATE;

        if (eventP->eType == keyDownEvent)
        {
            chr = keyTransfer(eventP->data.keyDown.chr, eventP->data.keyDown.keyCode, eventP->data.keyDown.modifiers);
            globeP->isKeyDownDectected = true;

            if (! (eventP->data.keyDown.modifiers & autoRepeatKeyMask))
            {
                if (globeP->isInPhoneView || (! settingP->shouldActivePluginInPhoneViewOnly))
                {
                    isKeyHandled = shouldKeyActivePlugin(chr, globeP);
                }

                if (! isKeyHandled)
                {
                    switch (chr)
                    {
                        case keyHard1:      //ż
                            if (! globeP->isInPhoneView)    //棬л绰
                            {
                                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                                formUpdateCode = FORM_UPDATE_ALL;
                                isKeyHandled = true;
                            }
                            break;
                        case keyHard4:      //680
                        case keyLaunch:     //
                            if
                            (
                                 settingP->useAsDefaultLauncher
                                 &&
                                 (
                                     (chr == keyLaunch)
                                     ||
                                     ((chr == keyHard4) && (settingP->deviceModel >= TREO_680))
                                 )
                            )
                            {
                                Boolean bKeepGoing = true;
                                UInt32  repeatTime = TimGetTicks() + 30;

                                while(KeyCurrentState() && bKeepGoing)
                                {
                                    if(TimGetTicks() >= repeatTime)
                                    {
                                        bKeepGoing = false;
                                    }
                                }

                                if (bKeepGoing)
                                {
                                    if ((eventP->data.keyDown.modifiers & poweredOnKeyMask) == 0) // ǰhomeĻر״̬
                                    {
                                        if (globeP->isInPhoneView)  //绰棬л
                                        {
                                            viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                                            formUpdateCode = FORM_UPDATE_ALL;
                                        }
                                        else if (CtlEnabled(globeP->ptriCategoryP)) //棬л
                                        {
                                            eventP->data.popSelect.priorSelection = LstGetSelection(globeP->lstCategoryP);
                                            if (eventP->data.popSelect.priorSelection != noListSelection && eventP->data.popSelect.priorSelection < LstGetNumberOfItems(globeP->lstCategoryP) - 1)
                                            {
                                                eventP->data.popSelect.selection = eventP->data.popSelect.priorSelection + 1;
                                            }
                                            else
                                            {
                                                if (settingP->shouldSkipAllCategory)
                                                {
                                                    eventP->data.popSelect.selection = 1;
                                                }
                                                else
                                                {
                                                    eventP->data.popSelect.selection = 0;
                                                }
                                            }

                                            LstSetSelection(globeP->lstCategoryP, eventP->data.popSelect.selection);
                                            eventP->eType = popSelectEvent;
                                            eventP->data.popSelect.controlID = ptriCategory;
                                            eventP->data.popSelect.controlP = globeP->ptriCategoryP;
                                            eventP->data.popSelect.listID = lstCategory;
                                            eventP->data.popSelect.listP = globeP->lstCategoryP;

                                            EvtAddEventToQueue(eventP);

                                            globeP->isPopSelectEventGeneratedByKey = true;

                                            //һл绰
                                            if (eventP->data.popSelect.priorSelection == LstGetNumberOfItems(globeP->lstCategoryP) - 1)
                                            {
                                                viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                                                formUpdateCode = FORM_UPDATE_ALL;
                                            }

                                        }
                                    }
                                    isKeyHandled = true;
                                }
                                else
                                {
                                    eventP->data.keyDown.chr = chrNull;
                                }
                            }
                            break;
                        case hsKeySymbol:
                            if (! globeP->isInPhoneView)
                            {
                                CtlHitControl(globeP->ptriCategoryP);
                                isKeyHandled = true;
                            }
                            break;
                        case chrUpArrow:        //
                        case vchrPageUp:
                        case vchrRockerUp:
                            isKeyHandled = arrowKeyEventHandler(chrUpArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case chrDownArrow:      //
                        case vchrPageDown:
                        case vchrRockerDown:
                            isKeyHandled = arrowKeyEventHandler(chrDownArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case chrLeftArrow:      //
                        case vchrRockerLeft:
                            isKeyHandled = arrowKeyEventHandler(chrLeftArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case chrRightArrow:     //
                        case vchrRockerRight:
                            isKeyHandled = arrowKeyEventHandler(chrRightArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case vchrRockerCenter:
                            if ((settingP->deviceModel == TREO_600 || settingP->deviceModel == NOT_A_TREO) || (! (eventP->data.keyDown.modifiers & willSendUpKeyMask)))
                            {
                                isKeyHandled = controlSelectEventHandler(globeP->focusedObjectId, settingP, globeP);
                            }
                            break;
                        case vchrMenu:  //ʾһЩ˵Ŀ
                            if (eventP->data.keyDown.chr == vchrMenu && eventP->data.keyDown.keyCode == 0 && eventP->data.keyDown.modifiers == commandKeyMask)
                            {
                                if (MenuGetActiveMenu() != NULL)
                                {
                                    showHideMenuitem(globeP);
                                }
                            }
                            break;
                        case keyBackspace:
                            if (globeP->searchKeywordNum > 0)
                            {
                                globeP->searchKeywordNum--;
                                globeP->searchKeyword[globeP->searchKeywordNum] = '\0';

                                if (globeP->searchKeywordNum == 0)
                                {
                                    if (globeP->isSearchBeginAtPhoneView)
                                    {
                                        viewSwitch(VIEW_MODE_PHONE, settingP, globeP);
                                    }
                                    else
                                    {
                                        viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                                    }
                                    formUpdateCode = FORM_UPDATE_ALL;
                                }
                                else
                                {
                                    if (settingP->shouldSearchInGlobe)
                                    {
                                        searchProgramInCategory(globeP->searchKeyword, ALL_CATEGORIES, globeP);
                                    }
                                    else
                                    {
                                        searchProgramInCategory(globeP->searchKeyword, settingP->lastCategory, globeP);
                                    }

                                    globeP->currentRow = 0;
                                    globeP->currentCol = 0;
                                    globeP->beginProgramInPage = 0;
                                    globeP->endProgramInPage = 11;

                                    if (globeP->programListCount == 0)
                                    {
                                        globeP->endProgramInPage = 0;
                                    }
                                    else if (globeP->endProgramInPage > globeP->programListCount - 1)
                                    {
                                        globeP->endProgramInPage = globeP->programListCount - 1;
                                    }

                                    formUpdateCode = FORM_UPDATE_ALL;
                                }
                            }
                            break;
                        default:
                            if ((chr >= keyZero && chr <= keyNine) ||
                                (eventP->data.keyDown.chr == eventP->data.keyDown.keyCode && eventP->data.keyDown.keyCode >= keyA && eventP->data.keyDown.keyCode <= keyZ) ||
                                chr == keySpace)
                            {
                                //û¿ʼ
                                if (settingP->shouldSearchProgramWhenKeyPressed && !settingP->shouldEnableHoldKeyRunProgram)
                                    keyPressEventHandler(eventP->data.keyDown.chr, eventP->data.keyDown.keyCode, &formUpdateCode, settingP, globeP);
                                    
                                isKeyHandled = true;
                            }
                            break;
                    }
                }
            }   // not repeat key
            else
            {
                if (chr >= keyA && chr <= keyZ)
                {
                    if (settingP->shouldEnableHoldKeyRunProgram)
                    {
                        globeP->holdedKey = chr;
                        drawKeyHoldTip(settingP);
                    }
                }
                else
                {
                    switch(chr)
                    {
                        case chrUpArrow:        //
                        case vchrPageUp:
                        case vchrRockerUp:
                            isKeyHandled = arrowKeyEventHandler(chrUpArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case chrDownArrow:      //
                        case vchrPageDown:
                        case vchrRockerDown:
                            isKeyHandled = arrowKeyEventHandler(chrDownArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case chrLeftArrow:      //
                        case vchrRockerLeft:
                            isKeyHandled = arrowKeyEventHandler(chrLeftArrow, &formUpdateCode, settingP, globeP);
                            break;
                        case chrRightArrow:     //
                        case vchrRockerRight:
                            isKeyHandled = arrowKeyEventHandler(chrRightArrow, &formUpdateCode, settingP, globeP);
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        else if (eventP->eType == keyUpEvent && globeP->isKeyDownDectected)
        {
            chr = keyTransfer(eventP->data.keyUp.chr, eventP->data.keyUp.keyCode, eventP->data.keyUp.modifiers);
            globeP->isKeyDownDectected = false;

            if (globeP->holdedKey != chrNull)   //ͷţ
            {
                if (globeP->holdedKey == chr && (chr >= keyA && chr <= keyZ))
                {
                    isShortCutExist = false;
                    if (! globeP->isInPhoneView)
                    {
                        getProgramListInCategory(ALL_CATEGORIES, globeP);   //ݼȫַΧ
                    }
                    for (selectedProgram = 0; selectedProgram < globeP->programListCount; selectedProgram ++)
                    {
                        if (eventP->data.keyHold.chr == globeP->programListP[selectedProgram].shortcut)
                        {
                            isShortCutExist = true;
                            launchProgram(&globeP->programListP[selectedProgram], selectedProgram, settingP, globeP);
                            break;
                        }
                    }
                    if (! isShortCutExist)
                    {
                        FrmCustomAlert(alertInformation, STRING_SHORTCUT_NOT_DEFINED, "", "");
                        if (! globeP->isInPhoneView)
                                {
                            getProgramListInCategory(settingP->lastCategory, globeP);   //ݼʧܣָǰб
                        }
                    }
                }
                globeP->holdedKey = chrNull;
            }
            else if (settingP->shouldEnableProfileManagement && chr == settingP->ProfileActiveKey)      //龰ģʽ
            {
                profileEventHandler(settingP, globeP);
            }
            else if ((! EvtKeydownIsVirtual(eventP)))   //򲦴绰
            {
                //б棬һոҳ绰л档
                if( eventP->data.keyUp.keyCode == keySpace)
                {
                    if (globeP->isInPhoneView)
                    {
                        viewSwitch(VIEW_MODE_PROGRAM, settingP, globeP);
                        formUpdateCode = FORM_UPDATE_ALL;
                    }
                    else if
                    (
                        globeP->programListCount > 0
                        &&
                        globeP->focusedObjectId == gdtIcons
                        &&
                        globeP->searchKeywordNum == 0
                    )
                    {
                        //ҳƶ3
                        if (globeP->endProgramInPage < globeP->programListCount - 1)
                        {
                            globeP->beginProgramInPage += 12;
                            globeP->endProgramInPage += 12;
                            if (globeP->endProgramInPage > globeP->programListCount - 1)
                            {
                                globeP->endProgramInPage = globeP->programListCount - 1;
                            }
                            formUpdateCode = FORM_UPDATE_LAUNCH_BAR_ONLY;
                        }
                        //кλ
                        if (globeP->beginProgramInPage + 4 * globeP->currentRow + globeP->currentCol > globeP->programListCount - 1)
                        {
                            globeP->currentRow = (globeP->endProgramInPage - globeP->beginProgramInPage) / 4;
                            globeP->currentCol = (globeP->endProgramInPage - globeP->beginProgramInPage) % 4;
                        }
                    }
                }
                else if (eventP->data.keyUp.keyCode == keyReturn)
                {
                    if
                    (
                        globeP->programListCount > 0
                        &&
                        globeP->focusedObjectId == gdtIcons
                    )
                    {
                        UInt16  objectId = globeP->beginProgramInPage + globeP->currentRow * 4 + globeP->currentCol;

                        launchProgram(&globeP->programListP[objectId], objectId, settingP, globeP);
                    }
                }
                else if (settingP->shouldSearchProgramWhenKeyPressed)   //
                {
                    //ǳô̰
                    if (settingP->shouldEnableHoldKeyRunProgram)
                        keyPressEventHandler(eventP->data.keyUp.chr, eventP->data.keyUp.keyCode, &formUpdateCode, settingP, globeP);
                }
                else    //򿪵绰
                {
                    if ((eventP->data.keyUp.keyCode >= keyA && eventP->data.keyUp.keyCode <= keyZ) ||
                        eventP->data.keyUp.keyCode == keySpace)
                    {
                        EvtEnqueueKey(eventP->data.keyUp.chr, eventP->data.keyUp.keyCode, 0x0800);
                    }
                    else if (eventP->data.keyUp.chr == keyZero)
                    {
                        EvtEnqueueKey(keyZero, 0xE020, 0x0800);
                    }

                    shouldKeyActivePlugin(keyHard1, globeP);
                }

                isKeyHandled = true;
            }
        }

        if (formUpdateCode != FORM_UPDATE_NO_UPDATE)
        {
            FrmUpdateForm(frmPhoneUI, formUpdateCode);
        }
    }

    return isKeyHandled;
}

//绰¼
static void frmPhoneUIMainEventHandler(SettingType *settingP)
{
    EventType       event;
    Int32           i;
    UInt16          error;
    UInt16          settingSize;
    UInt16          density         = kDensityDouble;
    GlobeType       *globeP;

    globeP = MemPtrNew(sizeof(GlobeType));

    if (globeP == NULL)
        return;

    MemSet(globeP, sizeof(GlobeType), 0);

    globeP->isVolumeMounted = isCardInserted();

    //ZLص
    globeP->ZLFuncPtr[ZLFuncPtrReopen] = NeoZLReopen;
    globeP->ZLFuncPtr[ZLFuncPtrBeginDraw] = NeoZLBeginDraw;
    globeP->ZLFuncPtr[ZLFuncPtrEndDraw] = NeoZLEndDraw;
    globeP->ZLFuncPtr[ZLFuncPtrPrepareHandlingEvent] = NeoZLPrepareHandlingEvent;
    globeP->ZLFuncPtr[ZLFuncPtrSetHint] = NeoZLSetHint;
    globeP->ZLFuncPtr[ZLFuncPtrShowHint] = NeoZLShowHint;
    globeP->ZLFuncPtr[ZLFuncPtrHideHint] = NeoZLHideHint;
    globeP->ZLFuncPtr[ZLFuncPtrPrefsReadValue] = NeoZLPrefsReadValue;
    globeP->ZLFuncPtr[ZLFuncPtrPrefsReadVariableValue] = NeoZLPrefsReadVariableValue;
    globeP->ZLFuncPtr[ZLFuncPtrPrefsWriteValue] = NeoZLPrefsWriteValue;
    globeP->ZLFuncPtr[ZLFuncPtrPrefsWriteVariableValue] = NeoZLPrefsWriteVariableValue;
    globeP->ZLFuncPtr[ZLFuncPtrRefreshBackground] = NeoZLRefreshBackground;
    globeP->ZLFuncPtr[ZLFuncPtrDrawChars] = NeoZLDrawChars;
    globeP->ZLFuncPtr[ZLFuncPtrDrawBitmap] = NeoZLDrawBitmap;

    //ȡİ汾
    settingSize = 0;
    i = PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, NULL, &settingSize, true);
    if (i != PREFERENCE_VERSION)    //һΰװ°汾
    {
        if (i != noPreferenceFound)     //°汾
        {
            //ɰ汾Ĳ
            PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, i, NULL, 0, true);
            i = noPreferenceFound;
        }
        else        //һΰװ
        {
            //б
            buildProgramList(BUILD_LIST_MODE_INIT, true, true, globeP);
        }
    }
    settingSize = sizeof(SettingType);
    if (i == noPreferenceFound)     //Ҫó
    {
        //ʼ
        MemSet(settingP, settingSize, 0);
        isRunningOnTreo(settingP);
        getDefaultColor(settingP);

        settingP->shouldSearchProgramWhenKeyPressed = true;

        settingP->shouldShowDateTime = true;
        settingP->shouldAddMask      = false;
        settingP->shouldShowTopIcons = true;

        settingP->maskPattern = whitePattern;

        settingP->shouldActivePluginInPhoneViewOnly = true;

        settingP->lastCategory = ALL_CATEGORIES;

        profileManagement(settingP, true);

        //
        PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, settingSize, true);
    }
    //ȡ
    PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, settingP, &settingSize, true);

    //ǷдӿҪɾĳ
    deleteProgramLaunchFromCard(settingP);

    //عб
    if (i != noPreferenceFound && settingP->shouldAutoUpdateProgramList)
    {
        buildProgramList(BUILD_LIST_MODE_REFRESH, true, false, globeP);
    }

    if (! settingP->useAsDefaultLauncher)
    {
        settingP->isNotificationRegistered = false;
    }

    //ʼȫֱ
    setRectangleData(globeP);
    getSoundStatusBitmap(settingP, globeP);
    if (settingP->shouldAddMask)
    {
        globeP->pluginParameter.defaultDisplayEffect = DISPLAY_EFFECT_ADD_MASK;
    }
    else if (settingP->shouldAddLine)
    {
        globeP->pluginParameter.defaultDisplayEffect = DISPLAY_EFFECT_ADD_LINE;
    }
    else
    {
        globeP->pluginParameter.defaultDisplayEffect = DISPLAY_EFFECT_NONE;
    }
    globeP->pluginParameter.defaultMaskPattern = settingP->maskPattern;
    globeP->pluginParameter.textColor = settingP->textColor;
    globeP->pluginParameter.textOutlineColor = settingP->textOutlineColor;
    globeP->pluginParameter.foreColor = settingP->formFrameColor;
    globeP->pluginParameter.backgdColor = settingP->formFillColor;

    // С
    loadTinyFont(globeP);

    //߷ֱĻּ֧
    if (WinGetSupportedDensity(&density) != errNone)
        globeP->isHighDensitySupported = false;
    else
        globeP->isHighDensitySupported = true;

    //ýɫ
    for (i = hsNavFocusColorRing1; i <= hsNavFocusColorRing6; i ++)
    {
        HsNavSetFocusColor(i, &settingP->focusRingColor[i - hsNavFocusColorRing1], NULL);
    }

    //ͼ
    globeP->offscreenWindowH = WinCreateOffscreenWindow(160, 160, nativeFormat, &error);

    if (error == errNone)
    {
        UInt32      value   = 0;

        //Gadget
//      SysLibLoad(kPmSysGadgetLibType, kPmSysGadgetLibCreator, &globeP->pmGadgetLibRef);
//      PmSysGadgetLibOpen(globeP->pmGadgetLibRef);

        //򿪵绰⣬ע绰¼
        HsGetPhoneLibrary(&globeP->hsPhoneLibRef);
        PhnLibOpen(globeP->hsPhoneLibRef);
        if (! settingP->isPhoneLibraryRegistered)
        {
            PhnLibRegister(globeP->hsPhoneLibRef, APPLICATION_CREATOR, phnServiceAll);
            settingP->isPhoneLibraryRegistered = true;
        }

        //
        SysLibLoad(tonesLibType, tonesLibCreator, &globeP->tonesLibRef);
        TonesLibOpen(globeP->tonesLibRef);

        //עϢ
        if (! settingP->isNotificationRegistered)
        {
            notifyRegister(settingP);
        }

        //򿪴
        FrmGotoForm(frmPhoneUI);

        do
        {
            EvtGetEvent(&event, evtWaitForever);

            if (! frmPhoneUICustomEventHandler(&event, settingP, globeP))
            {
                if (! frmPhoneUIKeyEventHandler(&event, settingP, globeP))
                {
                    if (! SysHandleEvent(&event))
                    {
                        if (! MenuHandleEvent(0, &event, &error))
                        {
                            if ((! frmPhoneUIAppEventHandler(&event, settingP, globeP)) && (! globeP->isMenuActived))
                            {
                                WinSetBackColorRGB(&settingP->topStatusFillColor, NULL);
                                FrmDispatchEvent(&event);
                                if ( globeP->isFormLoaded)
                                {
                                    if (shouldUpdateStatusBar(globeP))
                                        FrmUpdateForm(frmPhoneUI, FORM_UPDATE_STATUS_BAR_ONLY);
                                    if (shouldUpdateDateTime(globeP))
                                        FrmUpdateForm(frmPhoneUI, FORM_UPDATE_CONTENT);
                                }
                            }
                        }
                        else
                        {
                            //ǳ˵
                            if (event.data.keyDown.keyCode != vchrCommand)
                            {
                                globeP->isMenuActived = true;
                                globeP->isKeyDownDectected = false;
                            }
                        }
                    }
                }
            }
        }while (event.eType != appStopEvent);

        //ͷڴ
        freePluginList(globeP->pluginListP, globeP->pluginListCount);
        LstSetListChoices(globeP->lstCategoryP, NULL, 0);
        freeGlobeCategoryList(globeP);
        freeGlobeProgramList(globeP);
        if (globeP->bmpStatusP)
        {
            BmpDelete((BitmapType *)globeP->bmpStatusP);
        }
        if (globeP->bmpBackgdP)
        {
            BmpDelete((BitmapType *)globeP->bmpBackgdP);
        }

        //ر
        TonesLibClose(globeP->tonesLibRef);
        if (! settingP->useAsDefaultLauncher)
        {
            //ȡ绰¼ע
            if (settingP->isPhoneLibraryRegistered)
            {
                PhnLibRegister(globeP->hsPhoneLibRef, APPLICATION_CREATOR, 0);
                PhnLibClose(globeP->hsPhoneLibRef);
                settingP->isPhoneLibraryRegistered = false;
            }

            //ȡϢע
            notifyUnregister(settingP);
        }

        //رGadget
//      PmSysGadgetLibClose(globeP->pmGadgetLibRef);
    }

    if (globeP->offscreenWindowH)
        WinDeleteWindow(globeP->offscreenWindowH, false);
    for (i = 0; i < 3; i ++)
    {
        if (globeP->bmpSoundStatusP[i] != NULL)
            MemPtrFree(globeP->bmpSoundStatusP[i]);
    }
    for (i = 0; i < 2; i ++)
    {
        if (globeP->bmpPowerStatusP[i] != NULL)
            MemPtrFree(globeP->bmpPowerStatusP[i]);
    }
    for (i = 0; i < 3; i ++)
    {
        if (globeP->bmpBlueToothStatusP[i] != NULL)
            MemPtrFree(globeP->bmpBlueToothStatusP[i]);
    }
    if (globeP->bmpSearchP != NULL)
        MemPtrFree(globeP->bmpSearchP);

    // жС
    unloadTinyFont(globeP);

    MemPtrFree(globeP);

    //
    PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, settingSize, true);

    //رմ
    FrmCloseAllForms();
}

//绰
static void launchPhone()
{
    UInt16          settingSize = sizeof(SettingType);
    SettingType     *settingP   = MemPtrNew(settingSize);

    if (PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, settingP, &settingSize, true) != noPreferenceFound)
    {
        PluginParameterType     *pluginParameterP   = (PluginParameterType *)MemPtrNew(sizeof(PluginParameterType));

        MemSet(pluginParameterP, sizeof(PluginParameterType), 0);
        pluginParameterP->actionType = ACTION_TYPE_KEY;
        pluginParameterP->textColor = settingP->textColor;
        pluginParameterP->textOutlineColor = settingP->textOutlineColor;
        pluginParameterP->foreColor = settingP->formFrameColor;
        pluginParameterP->backgdColor = settingP->formFillColor;

        CallWithCommand('NlPh', APPLICATION_CREATOR, LAUNCH_CODE_PLUGIN_ACTION, pluginParameterP);

        MemPtrFree(pluginParameterP);
    }

    MemPtrFree(settingP);
}

//Ϣ¼
static void notifyEventHandler(SysNotifyParamType *notifyP)
{
    EventType           customEvent;

    customEvent.eType = nilEvent;
    switch (notifyP->notifyType)
    {
        /*case sysNotifyDBCreatedEvent:
        {
            buildProgramList(BUILD_LIST_MODE_ADD_DB, notifyP->notifyDetailsP);
            customEvent.eType = NOTIFY_EVENT_DB;
            break;
        }
        case sysNotifyDBChangedEvent:
        {
            buildProgramList(BUILD_LIST_MODE_CHANGE_DB, notifyP->notifyDetailsP);
            customEvent.eType = NOTIFY_EVENT_DB;
            break;
        }
        case sysNotifyDBDeletedEvent:
        {
            buildProgramList(BUILD_LIST_MODE_DELETE_DB, notifyP->notifyDetailsP);
            customEvent.eType = NOTIFY_EVENT_DB;
            break;
        }*/
        case sysNotifyVolumeMountedEvent:
        {
            notifyP->handled |= vfsHandledUIAppSwitch;
            customEvent.eType = NOTIFY_EVENT_CARD_MOUNT;
            break;
        }
        case sysNotifyVolumeUnmountedEvent:
        {
            customEvent.eType = NOTIFY_EVENT_CARD_UNMOUNT;
            break;
        }
        case sysNotifyLateWakeupEvent:
        {
            customEvent.eType = NOTIFY_EVENT_WAKE_UP;
            break;
        }
        case sysNotifyVirtualCharHandlingEvent:
        {
            if
            (
                ((SysNotifyVirtualCharHandlingType *)notifyP->notifyDetailsP)->keyDown.keyCode == keyHard1
                &&
                ((NotifyParameterType *)notifyP->userDataP)->shouldRelayKey
                &&
                ((NotifyParameterType *)notifyP->userDataP)->isInPhone == NOT_IN_CALL
            )
            {
                notifyP->handled = true;
                launchPhone();
            }
            else if
            (
                (
                    ((SysNotifyVirtualCharHandlingType *)notifyP->notifyDetailsP)->keyDown.keyCode == keyHard4
                    ||
                    ((SysNotifyVirtualCharHandlingType *)notifyP->notifyDetailsP)->keyDown.chr == keyHard4
                )
                &&
                ((NotifyParameterType *)notifyP->userDataP)->deviceModel >= TREO_680
                &&
                ((NotifyParameterType *)notifyP->userDataP)->useAsDefaultLauncher
            )
            {
                notifyP->handled = true;
                LaunchWithCommand('appl', APPLICATION_CREATOR, sysAppLaunchCmdNormalLaunch, NULL);
            }
            break;
        }
        case sysNotifyGotUsersAttention:
            customEvent.eType = NOTIFY_EVENT_ALERT;
            break;
    }

    if (customEvent.eType != nilEvent)
    {
        EvtAddUniqueEventToQueue(&customEvent, 0, false);
    }
}

#if 0
#pragma mark -
#pragma mark START UP FUNCTIONS
#pragma mark -
#endif

//豸
static Boolean isRunningOnTreo(SettingType *settingP)
{
    UInt32          companyID;
    UInt32          deviceID;

    FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &companyID);
    FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &deviceID);

    if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D053') //Treo680
    {
        settingP->deviceModel = TREO_680;
    }
    else if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D061') //CENTRO
    {
        settingP->deviceModel = CENTRO;
    }
    else if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D052') //Treo700p
    {
        settingP->deviceModel = TREO_700P;
    }
    else if ((companyID == 'Palm' || companyID == 'hspr') && deviceID == 'D060') //Treo755p
    {
        settingP->deviceModel = TREO_755P;
    }
    else if (companyID == 'hspr' && (deviceID == 'H102' || deviceID == 'H202')) //Treo650
    {
        settingP->deviceModel = TREO_650;
    }
    else if (companyID == 'hspr' && (deviceID == 'H101' || deviceID == 'H201')) //Treo600
    {
        settingP->deviceModel = TREO_600;
    }
    else
    {
        settingP->deviceModel = NOT_A_TREO;
        return false;
    }

    return true;
}


//ڵ
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    SettingType         *settingP;
    UInt16              i;
    PhnEventType        *phnEventP;

    settingP = MemPtrNew(sizeof(SettingType));
    if (settingP == NULL)
        return memErrorClass;

    switch (cmd)
    {
        case sysAppLaunchCmdNormalLaunch:
        case sysAppLaunchCmdReturnFromPanel:
            if (isRunningOnTreo(settingP))
            {
                frmPhoneUIMainEventHandler(settingP);
            }
            else
            {
                FrmCustomAlert(alertError, STRING_DEVICE_ERROR, "", "");
            }
            break;
        case sysAppLaunchCmdSystemReset:
            i = sizeof(SettingType);
            if (PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, settingP, &i, true) != noPreferenceFound)
            {
                settingP->isNotificationRegistered = false;
                settingP->isPhoneLibraryRegistered = false;
                PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, sizeof(SettingType), true);
            }
            break;
        case sysAppLaunchCmdNotify:
            notifyEventHandler((SysNotifyParamType *)cmdPBP);
            break;
        case sysAppLaunchCmdAlarmTriggered:
            break;
        case sysAppLaunchCmdDisplayAlarm:
            break;
        case phnLibLaunchCmdEvent:
        {
            EventType               customEvent;
            NotifyParameterType     *notifyParameterP;

            phnEventP = (PhnEventType *)cmdPBP;
            MemSet(&customEvent, sizeof(EventType), 0);
            switch (phnEventP->eventType)
            {
                case phnEvtRegistration:
                    switch (phnEventP->data.registration.status)
                    {
                        case registrationHome:
                            customEvent.eType = PHONE_EVENT_HOME;
                            break;
                        case registrationSearch:
                            customEvent.eType = PHONE_EVENT_NETWORK_SEARCHING;
                            break;
                    }
                    break;

                case phnEvtPower:
                    switch (phnEventP->data.power.state)
                    {
                        case phnPowerOn:
                            customEvent.eType = PHONE_EVENT_POWER_ON;
                            break;
                        case phnPowerOff:
                            customEvent.eType = PHONE_EVENT_POWER_OFF;
                            break;
                    }
                    break;
                case phnEvtIndication:
                    switch (phnEventP->data.indication.kind)
                    {
                        case indicationNetworkSearch:
                            customEvent.eType = PHONE_EVENT_NETWORK_SEARCHING;
                            break;
                        case indicationStartingRadio:
                            customEvent.eType = PHONE_EVENT_POWER_ON;
                            break;
                        case indicationPoweringOffRadio:
                            customEvent.eType = PHONE_EVENT_POWER_OFF;
                            break;
                    }
                    break;
                case phnEvtStartDial:
                case phnEvtAlertingPreConnected:
                    if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) == errNone)
                    {
                        NotifyParameterType     tmpNotifyParameter  = (*notifyParameterP);

                        tmpNotifyParameter.isInPhone = OUTGOING_CALL;   //绰
                        DmWrite(notifyParameterP, 0, &tmpNotifyParameter, sizeof(NotifyParameterType));
                    }
                    break;
                case phnEvtStartIncomingCall:
                    if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) == errNone)
                    {
                        NotifyParameterType     tmpNotifyParameter  = (*notifyParameterP);

                        tmpNotifyParameter.isInPhone = INCOMING_CALL;   //绰
                        DmWrite(notifyParameterP, 0, &tmpNotifyParameter, sizeof(NotifyParameterType));
                    }
                    break;
                case phnEvtDisconnectInd:
                case phnEvtDisconnectConf:
                    {
                        UInt8   isInPhone = INCOMING_CALL;

                        if (FtrGet(APPLICATION_CREATOR, NOTIFY_PARAMETER_FEATURE_ID, (UInt32 *)&notifyParameterP) == errNone)
                        {
                            NotifyParameterType     tmpNotifyParameter;

                            tmpNotifyParameter  = (*notifyParameterP);
                            isInPhone = tmpNotifyParameter.isInPhone;   //ͨ״̬
                            tmpNotifyParameter.isInPhone = NOT_IN_CALL; //ͨ
                            DmWrite(notifyParameterP, 0, &tmpNotifyParameter, sizeof(NotifyParameterType));
                        }
                        i = sizeof(SettingType);
                        if (PrefGetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, settingP, &i, true) != noPreferenceFound)
                        {
                            //߽绰NL
                            if ((settingP->ReturnToLauncherAfterCall & isInPhone) != 0)
                            {
                                LaunchWithCommand('appl', APPLICATION_CREATOR, sysAppLaunchCmdNormalLaunch, NULL);
                            }
                        }

                    }
                    break;
            }
            if (customEvent.eType != nilEvent)
            {
                //EvtAddEventToQueue(&customEvent);
                EvtAddUniqueEventToQueue(&customEvent, 0, false);
            }
            break;
        }
    }

    MemPtrFree(settingP);

    return errNone;
}

#if 0
#pragma mark -
#pragma mark COMMON FUNCTIONS
#pragma mark -
#endif

//λ
static void raiseProgram(ProgramListType *programP)
{
    if (! (programP->addOnInfo & PROGRAM_LIST_STATIC_MASK)) //ǹ̶
    {
        UInt16          currentProgramIndex;
        UInt16          newProgramIndex;
        UInt16          i;
        MemHandle       recordH;
        ProgramListType *recordP;
        UInt16          lastStaticProgramIndex  = 0xFFFF;
        DmOpenRef       dbRef                   = DmOpenDatabaseByTypeCreator(PROGRAM_LIST_DB_TYPE, APPLICATION_CREATOR, dmModeReadWrite);
        UInt16          programCount            = DmNumRecordsInCategory(dbRef, dmAllCategories);

        //ȡǰбеλãԼһ̶λ
        for (i = 0; i < programCount; i ++)
        {
            recordH = DmQueryRecord(dbRef, i);
            recordP = (ProgramListType *)MemHandleLock(recordH);

            if ((recordP->addOnInfo & PROGRAM_LIST_STATIC_MASK))
            {
                lastStaticProgramIndex = i;
            }

            if (recordP->type == programP->type && recordP->creatorId == programP->creatorId)
            {
                currentProgramIndex = i;
                MemHandleUnlock(recordH);
                break;
            }

            MemHandleUnlock(recordH);
        }

        if ((lastStaticProgramIndex != 0xFFFF && currentProgramIndex - 1 > lastStaticProgramIndex) ||
            (lastStaticProgramIndex == 0xFFFF && currentProgramIndex > 0))  //Եλ
        {
            if (lastStaticProgramIndex != 0xFFFF)
            {
                newProgramIndex = currentProgramIndex - (currentProgramIndex - lastStaticProgramIndex) / 2;
            }
            else
            {
                newProgramIndex = currentProgramIndex / 2;
            }
            DmMoveRecord(dbRef, currentProgramIndex, newProgramIndex);
        }

        DmCloseDatabase(dbRef);
    }
}

//
static void launchProgram(ProgramListType *programInfoP, UInt16 selectedProgram, SettingType *settingP, GlobeType *globeP)
{
    //̬λ
    if (settingP->shouldAutoRaiseProgram)
        raiseProgram(programInfoP);

    if (programInfoP->VFSNameP[0] == '\0')  //ڴеĳ
    {
        LaunchWithCommand(programInfoP->type, programInfoP->creatorId, sysAppLaunchCmdNormalLaunch, NULL);
    }
    else
    {
        Err         errorCode;
        UInt16      cardNo;
        LocalID     dbId;
        UInt16      volRef          = 0;
        UInt32      volIterator     = vfsIteratorStart;

        //ȡָ
        while (volIterator != vfsIteratorStop)
        {
            VFSVolumeEnumerate(&volRef, &volIterator);
        }

        if (volRef > 0)
        {
            StrCopy(globeP->cacheP, "/PALM/Launcher/");
            StrCat(globeP->cacheP, programInfoP->VFSNameP);
            errorCode = VFSImportDatabaseFromFile(volRef, globeP->cacheP, &cardNo, &dbId);
            if (errorCode != expErrNotEnoughPower && errorCode != vfsErrBadName)
            {
                if (errorCode != dmErrAlreadyExists)    //¼
                {
                    if (DmDatabaseInfo(cardNo, dbId, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &settingP->lastProgramLaunchFromCard.type, &settingP->lastProgramLaunchFromCard.creatorId) == errNone)
                    {
                        PrefSetAppPreferences(APPLICATION_CREATOR, PREFERENCE_ID, PREFERENCE_VERSION, settingP, sizeof(SettingType), true);
                    }
                }
                SysUIAppSwitch(cardNo, dbId, sysAppLaunchCmdNormalLaunch, NULL);
            }
            else
            {
                FrmCustomAlert(alertError, STRING_LAUNCH_ERROR, "", "");
            }
        }
    }
}

//
static void paintCharWithOutline(Char chr, Coord x, Coord y, Boolean shouldDrawOutline, RGBColorType *outlineColorP, GlobeType *globeP)
{
    if (shouldDrawOutline)
    {
        RGBColorType    defaultColor;

        //
        WinSetTextColorRGB(outlineColorP, &defaultColor);
        if (globeP->isHighDensitySupported)    //߷Ļ
        {
            Coord   dX = x * 2;
            Coord   dY = y * 2;

            WinSetCoordinateSystem(kCoordinatesDouble);
            WinPaintChar(chr, dX, dY - 1);
            WinPaintChar(chr, dX, dY + 1);
            WinPaintChar(chr, dX - 1, dY);            
            WinPaintChar(chr, dX + 1, dY);
            WinPaintChar(chr, dX - 1, dY-1);
            WinPaintChar(chr, dX - 1, dY+1);
            WinPaintChar(chr, dX + 1, dY-1);
            WinPaintChar(chr, dX + 1, dY+1);
            WinSetCoordinateSystem(kCoordinatesStandard);
        }
        else    //ָ֧߷ֱĻ
        {
            WinPaintChar(chr, x - 1, y - 1);
            WinPaintChar(chr, x + 1, y - 1);
            WinPaintChar(chr, x - 1, y + 1);
            WinPaintChar(chr, x + 1, y + 1);
        }
        WinSetTextColorRGB(&defaultColor, NULL);
    }

    //ַ
    WinPaintChar(chr, x, y);
}

//ַ
static void paintCharsWithOutline(const Char *stringP, UInt16 stringLength, Coord x, Coord y, Boolean shouldDrawOutline, RGBColorType *outlineColorP, GlobeType *globeP)
{
    if (shouldDrawOutline)
    {
        RGBColorType    defaultColor;

        //
        WinSetTextColorRGB(outlineColorP, &defaultColor);
        if (globeP->isHighDensitySupported)     //߷Ļ
        {
            Coord   dX = x * 2;
            Coord   dY = y * 2;

            WinSetCoordinateSystem(kCoordinatesDouble);
            WinPaintChars(stringP, stringLength, dX - 1, dY - 1);
            WinPaintChars(stringP, stringLength, dX + 1, dY - 1);
            WinPaintChars(stringP, stringLength, dX - 1, dY + 1);
            WinPaintChars(stringP, stringLength, dX + 1, dY + 1);
            WinSetCoordinateSystem(kCoordinatesStandard);
        }
        else    //ָ֧߷ֱĻ
        {
            WinPaintChars(stringP, stringLength, x - 1, y - 1);
            WinPaintChars(stringP, stringLength, x + 1, y - 1);
            WinPaintChars(stringP, stringLength, x - 1, y + 1);
            WinPaintChars(stringP, stringLength, x + 1, y + 1);
        }
        WinSetTextColorRGB(&defaultColor, NULL);
    }

    //ַ
    WinPaintChars(stringP, stringLength, x, y);
}

//ָȵ
static void winPaintTruncChars(Char *stringP, UInt16 stringLength, Coord x, Coord y, UInt16 maxWidth, Boolean shouldDrawOutline, RGBColorType *outlineColorP, GlobeType *globeP)
{
    if (stringLength > 3 && stringLength < 128)
    {
        StrCopy(globeP->cacheP, stringP);
        if ((UInt8)globeP->cacheP[stringLength - 2] > 127)  //
        {
            StrCopy((globeP->cacheP + (stringLength - 2)), "...");
            stringLength ++;
        }
        else
        {
            StrCopy((globeP->cacheP + (stringLength - 1)), "...");
            stringLength += 2;
        }

        while (stringLength > 3)
        {
            if (FntCharsWidth(globeP->cacheP, stringLength) <= maxWidth)
            {
                if (shouldDrawOutline)
                {
                    paintCharsWithOutline(globeP->cacheP, stringLength, x, y, true, outlineColorP, globeP);
                }
                else
                {
                    WinPaintChars(globeP->cacheP, stringLength, x, y);
                }
                break;
            }
            else
            {
                if ((UInt8)globeP->cacheP[stringLength - 5] > 127)
                {
                    StrCopy((globeP->cacheP + (stringLength - 5)), "...");
                    stringLength -= 2;
                }
                else
                {
                    StrCopy((globeP->cacheP + (stringLength - 4)), "...");
                    stringLength --;
                }
            }
        }
    }
}

//ַתUInt32λװ
static void convertCharsToBitPackage(Char *srcStringP, UInt32 *destUInt32BitPackageP, UInt16 *UInt32BitPackageLengthP)
{
    WChar       chr;

    (*UInt32BitPackageLengthP) = 0;

    while (*srcStringP && (*UInt32BitPackageLengthP) < 5)
    {
        if (((UInt8)(*srcStringP)) > 127)   //
        {
            Char ch;

            MemMove(&chr, srcStringP, 2);
            srcStringP += 2;

            if (chr < 0xB0C5) ch= 'a';
            else if (chr < 0xB2C1) ch= 'b';
            else if (chr < 0xB4EE) ch= 'c';
            else if (chr < 0xB6EA) ch= 'd';
            else if (chr < 0xB7A2) ch= 'e';
            else if (chr < 0xB8C1) ch= 'f';
            else if (chr < 0xB9FE) ch= 'g';
            else if (chr < 0xBBF7) ch= 'h';
            else if (chr < 0xBFA6) ch= 'j';
            else if (chr < 0xC0AC) ch= 'k';
            else if (chr < 0xC2E8) ch= 'l';
            else if (chr < 0xC4C3) ch= 'm';
            else if (chr < 0xC5B6) ch= 'n';
            else if (chr < 0xC5BE) ch= 'o';
            else if (chr < 0xC6DA) ch= 'p';
            else if (chr < 0xC8BB) ch= 'q';
            else if (chr < 0xC8F6) ch= 'r';
            else if (chr < 0xCBFA) ch= 's';
            else if (chr < 0xCDDA) ch= 't';
            else if (chr < 0xCEF4) ch= 'w';
            else if (chr < 0xD1B9) ch= 'x';
            else if (chr < 0xD4D1) ch= 'y';
            else  ch= 'z';

            destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (((UInt32)1) << (ch - 97));
        }
        else if (((UInt8)(*srcStringP)) >= 97 && ((UInt8)(*srcStringP)) <= 122) //СдӢ
        {
            destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (((UInt32)1) << ((*srcStringP) - 97));
            srcStringP ++;
        }
        else if (((UInt8)(*srcStringP)) >= 65 && ((UInt8)(*srcStringP)) <= 90)  //дӢ
        {
            destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = (((UInt32)1) << ((*srcStringP) - 65));
            srcStringP ++;
        }
        else    //
        {
            destUInt32BitPackageP[(*UInt32BitPackageLengthP)] = 0x80000000;
            srcStringP ++;
        }
        (*UInt32BitPackageLengthP) ++;
    }
}
